// By Scott Dunlop

String hasSlot("splitLines") ifNil(
    String splitLines := method( 
        lineStart := 0
        resultList := List clone

        nextCrIndex := Nil
        nextLfIndex := Nil

        while( lineStart < size,
            nextCrIndex = find( "\r", lineStart )
            nextLfIndex = find( "\n", lineStart )

            if( nextCrIndex and nextLfIndex )then(
                if( nextCrIndex < nextLfIndex )then(
                    nextLfIndex = Nil
                )else(
                    nextCrIndex = Nil
                )
            )

            if( nextCrIndex )then(
                resultList add( fromTo( lineStart, nextCrIndex ) )
                if( at( nextCrIndex + 1 ) == 10 )then(
                    lineStart = nextCrIndex + 2
                )else(
                    lineStart = nextCrIndex + 1
                )
            )elseif( nextLfIndex )then(
                resultList add( fromTo( lineStart, nextLfIndex ) )
                lineStart = nextLfIndex + 1
            )else(
                break
            )
        )

        resultList add( fromTo( lineStart, size ) )

        return resultList
    )
)

HttpServer := Object clone do(
    type := "HttpServer"
    HttpRequest := Object clone
    HttpRequest do (
            lineMode := 1

            mandatorySpacePattern := "[ \t]+"
            optionalSpacePattern := "[ \t]*"
            tokenPattern := "([^\\x00-\\x1F\\x7F()<>@.;:\\\\\"/[\\]?={} \t]+)"
            quotedPattern := "\"((?:\\\\\"|[^\"])+\)\""

            headerPattern := "^" .. tokenPattern .. optionalSpacePattern .. ":" .. optionalSpacePattern .. "(.*)$"
            commandPattern := "^" .. tokenPattern .. mandatorySpacePattern .. "(.*)$" 
            pairPattern := "^" .. optionalSpacePattern .. tokenPattern .. optionalSpacePattern .. "=" .. optionalSpacePattern .. "(?:" .. tokenPattern .. "|" .. quotedPattern .. ")(?:;(.*))?$"
            queryPattern := "^([^?]*).(.*)$"

            headerRegex := Regex clone setPattern( headerPattern )
            commandRegex := Regex clone setPattern( commandPattern )
            pairRegex := Regex clone setPattern( pairPattern )
            queryRegex := Regex clone setPattern( queryPattern )

            init := method(
                self sentCookies := Map clone
            )

            lineBuffer := method(
                self lineBuffer := Buffer clone
            )
            rawHeaders := method(
                self rawHeaders := List clone
            )
            headers := method(
                self headers := Map clone
                self rawHeaders foreach(i, v,
                    self headers atPut( v at(0)asString, v at(1)asString)
                )
                return self headers
            )
            sentBuffer := method(
                self sentBuffer := Buffer clone
            )
            queryArgs := method(
                self parseQuery; return queryArgs
            )
            queryPath := method(
                self parseQuery; return queryPath
            )
            queryCookies := method(
                self parseQueryCookies; return queryCookies
            )
            handleSocket := method( aSocket,
                self mySocket := aSocket

                while( self mySocket isOpen,
                    if( self mySocket read )then(
                        self handleInput( self mySocket readBuffer )
                    )
                )
            )
            handleInput := method( readBuffer,
                if( lineMode )then(
                    lineBuffer appendSeq( readBuffer )
                    readBuffer empty

                    readLines := lineBuffer asString splitLines

                    rest := readLines pop

                    readLines foreach( i, line, self currentParser( line ) )

                    if( lineMode )then(
                        self lineBuffer := rest asBuffer
                    )else(
                        self currentParser( rest asString )
                    )
                )else(
                    self lineBuffer appendSeq( readBuffer )
                    self currentParser( readBuffer asString )
                )
            )
            handleRequest := method( request,
                self close
            )

            currentParser := method( line,
                self chainParser( "commandParser", line )
            )
            chainParser := method( parserName, line,
                self currentParser := self getSlot( parserName )
                self currentParser( line )
            )
            commandParser := method( line,
                self commandRegex setString( line )
                if( self commandRegex hasMatch )then(
                    command := self commandRegex firstMatch
                    self command := command at(1)
                    self arguments := command at(2) split(" ")
                )else(
                    self chainParser( "headerParser", line )
                )
            )
            headerParser := method( line,
                self headerRegex setString( line )
                if( self headerRegex hasMatch )then(
                    header := self headerRegex firstMatch
                    self rawHeaders add(
                        List clone add(header at(1), header at(2))
                    )
                )else(
                    self chainParser( "completeParser", line )
                )
            )
            completeParser := method( line,
                if( line size == 0 )then(
                    self determineCommand
                )else(
                    Lobby write( 
                        "Server does not understand http command: '", 
                        line, "'\n" 
                    )
                )
            )
            determineCommand := method(
                if( command == "GET" )then( 
                    self handleRequest( self )
                )else(
                    if( command == "POST" )then( 
                        self lineBuffer empty
                        self currentParser := self getSlot( "formParser" )
                        self contentLength := self headers at( 
                            "Content-Length" 
                        )
                        self lineMode = Nil
                    )
                )
            )
            formParser := method( data,
                lineBuffer appendSeq( data )
                if( lineBuffer size >= contentLength )then(
                    self queryPath := self arguments at( 0 )
                    self queryArgs := CGI parseString( 
                        lineBuffer asString
                    )
                    self handleRequest( self )
                    lineBuffer empty
                )
            )
            parseQuery := method(
                // This method is invoked lazily by queryPath and queryArgs
                query := self arguments at( 0 )asString
                if( query find("?") )then(
                    self queryRegex setString( query )
                    self queryPath := self queryRegex firstMatch at( 1 )
                    self queryArgs := CGI parseString( 
                        self queryRegex firstMatch at( 2 ) 
                    )
                )else(
                    self queryPath := query 
                    self queryArgs := Map clone
                )
            )
            parseQueryCookies := method(
                cookieHeader := self headers at( "Cookie" )

                self queryCookies := Map clone

                while( 1,
                    cookieHeader ifNil( return )

                    results := self pairRegex setString( 
                        cookieHeader 
                    )firstMatch
                    results ifNil( return )

                    if( results at( 2 ) == "" )then(
                        self queryCookies atPut( 
                            results at( 1 ), results at( 3 ) 
                        )
                    )else(
                        self queryCookies atPut( 
                            results at( 1 ), results at( 2 ) 
                        )
                    )
                    cookieHeader := results at( 4 )
                )
            )
            sendCookie := method( key, value, 
                sentCookies atPut( key, value )
            )
            sendList := method( data,
                data foreach( i, v, self sentBuffer appendSeq( v ) )
            )
            send := method(            	
                sendList( thisMessage argsEvaluatedIn( sender ) )
            )
            sendHeader := method( key, value,
                self send( key, ": ", value, "\r\n" )
            )
            endHeaders := method(
                self send( "\r\n" )
            )
            sendResponse := method( code, message,
                self responseCode := code
                self responseMessage := message
            )
            flush := method(
                self removeSlot( "sentBuffer" )
            )
            done := method(
                self mySocket write( 
                    "HTTP/1.1 ", self responseCode asString, " ", 
                    self responseMessage, "\r\n" 
                )
                self sentCookies foreach( key, value,
                    self mySocket write( 
                        "Set-cookie: ", key, "=", value, "\r\n" 
                    )
                )
                self mySocket write( sentBuffer asString )
                self flush
            )
            close := method(
                self hasSlot( "sentBuffer" )ifTrue( self done )
                self mySocket close
            )
        )

        serve := method(httpPort,
            if( thisMessage arguments count == 2)then(
                // We've been given a block.
                requestHandler := thisMessage arguments at(1)
            ) else (
                // We assume we've been given a slot-name and a message.
                requestHandler := block( Nil )
                getSlot("requestHandler")setArgumentNames(
                    List clone add( thisMessage arguments at(1) name )
                )
                getSlot("requestHandler")setMessage(
                    thisMessage arguments at(2) 
                )
                getSlot("requestHandler")setScope( sender )
            )

            requestProto := HttpRequest clone
            requestProto handleRequest := getSlot("requestHandler")

            return self serveWithProto( httpPort, requestProto )
        )

        serveWithProto := method(httpPort, requestProto,
            server := Server clone setPort( httpPort )
            server requestProto := requestProto
            server handleSocket := method( aSocket,
		aSocket setBytesPerRead( 10000000 )
                requestProto clone handleSocket( aSocket )
            )
            server @start
            return server
        )
)

//A really simple, and terminally rude, http server:
/*
HttpServer serve( 8080, req, 
    req sendResponse( 200, "OK" )
    req sendHeader( "Content-type", "text/plain" )
    req endHeaders() 
    req send( req queryPath )
    req close
) 
*/

