// By Jason Collins
// -----------------------------------------------------------------------
doFile(Path with(launchPath, "CursesScreen.io"))

ChatView := Object clone do(
	outputLines := List clone
	
	addOutput := method(s,
		s asString replace("\r", "") split("\n") foreach(line,
		  outputLines append(line)
		)
		while(outputLines size > 200, outputLines removeAt(0))
		draw
 	)
	
	input := ""
	setInput := method(s, input = s; draw)

	draw := method(
		w := Curses width
		h := Curses height
		Curses clear
		Curses move(0,0)

		p := h - 3
		outputLines reverseForeach(i, line,
		    Curses move(p, 0)
		    Curses print(line)
		    p = p - 1
		    if (p < 0, break)
		)

		Curses move(h-2,0)
		w repeatTimes(Curses print("-"))

		Curses move(h-1,0)
		Curses print(input)

		Curses refresh
	)
)


// -----------------------------------------------------


IRCClient := Object clone do(

	dataSock := Nil
	writeBuffer := Buffer clone
	nickName := "joeshmo"
	curChannel := ""
	chat := ChatView



	connectTo := method(server, port,
	//
	//Opens the I/O socket and does ident listening
	//TODO: implement ident
	//
		dataSock = Socket clone setHost(server) setPort(port)
		server foreach(i, v,
			if (v isLetter,
				serverIp := Host clone setName(server) address
				chat addOutput(server .. " resolved to " .. serverIp .. "\n")
				dataSock setHost(serverIp)
				break
			)
		)
		dataSock connect
		chat addOutput("Socket connected on port " .. port asString .. "\n")
		if (dataSock error, chat addOutput(dataSock error .. "\n"); exit)
		self
	)


	readServerMessage := method(buf,
	//
	//Parse qualified message from the irc server
	//
		bufSlice := buf
		while (bufSlice asString find("\r\n", 0) != Nil,
			cmd := bufSlice asString
			bufSlice = bufSlice fromTo(cmd find("\r\n") + 2, -1)
			cmd = cmd substring(0, cmd find("\r\n"))

			senderString := cmd substring(1, cmd find(" "))
			senderNick := "[server]"
			if (senderString find("!") != Nil, senderNick = senderString substring(0, senderString find("!")))
			cmd = cmd substring(cmd find(" ") + 1)

			preMsg := cmd substring(0, cmd find(" "))
			cmd = cmd substring(cmd find(" ") + 1)

			if (preMsg == "NOTICE" or preMsg == "PRIVMSG",
				msgPart := cmd substring(cmd find(":") + 1)
				if (cmd at(0) == "#" at(0),
					chat addOutput("<" .. senderNick .. "> " .. msgPart .. "\n")
				,
					chat addOutput("[From " .. senderNick .. "]: " .. msgPart .. "\n")
				)
				continue
			)

			if (preMsg == "JOIN",
				cmd = cmd substring(cmd find(":") + 1)
				if (senderNick == nickName,
					chat addOutput("*** You joined channel " .. cmd .. " ***\n")
					curChannel = cmd
				,
					chat addOutput("*** " .. senderNick .. " joined channel " .. cmd .. " ***\n")
				)
				continue
			)
		)

		buf empty
		if (bufSlice size > 0, buf appendSeq(bufSlice))
	)

	sendWriteBuffer := method(
		if (writeBuffer at(0) == "/" at(0)) then (
			writeBuffer = writeBuffer fromTo(1, writeBuffer size)
			dataSock write(writeBuffer, "\r\n")
		) else (
			chat addOutput("<" .. nickName .. "> " .. writeBuffer)
			dataSock write("PRIVMSG " .. curChannel .. " :" .. writeBuffer .. "\r\n")
		)
		writeBuffer empty
	)

	checkWrite := method(
	//
	//Looks to see if we have incoming keyboard data, and if so appends it
	//to the socket's outgoing buffer.  If a carriage return is read, then
	//the buffer is packaged and written to the socket
	//
		ch := Curses getCh
		if (ch != Nil,
			if (ch == 13, sendWriteBuffer, writeBuffer appendSeq(ch asCharacter))
			chat setInput(writeBuffer)
		)
	)


	updateSocket := block(
	//
	//Performs polling loop on the incoming socket
	//
		while (dataSock isOpen,
			if(dataSock read, try(readServerMessage(dataSock readBuffer)) catch(e, exit), yield)
		)
	)



	getInputLine := method(
		b := Buffer clone
		while(1,
			c := Curses getCh
			if (c == 13, return b)
			if (c != Nil and c < 128, 
				b appendSeq(c asCharacter)
				Curses print(c asCharacter)
				Curses refresh
			)
		)
	)

	connect := method(
		//connectTo("irc.enterthegame.com", 6667)
		connectTo("69.72.218.59", 6667)
		@@updateSocket
		dataSock write("NICK ", nickName, "\r\n")
		dataSock write("USER collin 0 0 bob@shmob.cob\r\n")
		dataSock write("JOIN #jason\r\n")	
	)
)

// ----------------------------------------


CursesScreen setup
//CursesScreen addSubview(ChatView)
CursesScreen eventLoop
