/*#io
Nil ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Nil")
docDescription("""Nil is a singleton object that is used as a placeholder and to mean "false" in Io.""")
*/

#include "IoNil.h"
#include "IoState.h"
#include "IoObject.h"
#include "IoSeq.h"
#include "IoMessage.h"

IoTag *IoNil_tag(void *state)
{
    IoTag *tag = IoTag_newWithName_("Nil");
    tag->state = state;
    tag->cloneFunc = (TagCloneFunc *)IoNil_rawClone;
    return tag;
}

IoObject *IoNil_proto(void *state)
{
    IoMethodTable methodTable[] = {
    {"clone", IoObject_self},
    /*#io
    docSlot("clone", 
    "returns self. (Nil is a singleton)")
    */
    {"and", IoNil_and},
    {"or", IoNil_or},
    {"print", IoNil_print},
    {"==", IoNil_isEqual},
    {"!=", IoNil_isNotEqual},
    {"then", IoObject_nop},
    {"else", IoObject_evalArg},
    {"elseif", IoObject_if},
    {"isNil", IoNil_isNil},
    {"ifNil", IoNil_ifNil},
    {"ifFalse", IoNil_ifFalse},
    {"ifTrue", IoNil_ifTrue},
    {NULL, NULL},
    };
    
    IoObject *self = IoObject_new(state);
    self->tag = IoNil_tag(state);
    IoState_registerProtoWithFunc_(state, self, IoNil_proto);
    
    IoObject_addMethodTable_(self, methodTable);
    return self;
}

/* carefull - the compiler might not make this unique */
IoObject *IoNil_rawClone(IoObject *self) 
{ 
    return self; 
} 

IoObject *IoNil_new(void *state) 
{ 
    return ((IoState *)state)->ioNil; 
}

void IoNil_printNil(IoObject *self) 
{ 
    IoState_print_(IOSTATE, "Nil"); 
}

IoObject *IoNil_and(IoObject *self, IoObject *locals, IoMessage *m)
{ 
    /*#io
    docSlot("and anObject", 
    "Returns Nil.")
    */
    
    return self; 
}

IoObject *IoNil_or(IoObject *self, IoObject *locals, IoMessage *m)
{ 
    /*#io
    docSlot("or anObject", 
    "Returns anObject if anObject is not Nil. Otherwise returns Nil.")
    */
    
    IoObject *other = IoMessage_locals_valueArgAt_(m, locals, 0);
    return (!ISNIL(other)) ? (IoObject *)other : self; 
}

IoObject *IoNil_print(IoObject *self, IoObject *locals, IoMessage *m)
{
    /*#io
    docSlot("print", 
    "Prints 'Nil'. Returns self.")
    */
    
    IoState_print_(IOSTATE, "Nil");
    return self;
}

IoObject *IoNil_isEqual(IoObject *self, IoObject *locals, IoMessage *m)
{
    IoObject *other = IoMessage_locals_valueArgAt_(m, locals, 0);
    
    if (other == self) 
    { 
	return ((IoState *)IOSTATE)->lobby; 
    }
    
    return self;
}

IoObject *IoNil_isNotEqual(IoObject *self, IoObject *locals, IoMessage *m)
{
    IoObject *other = IoMessage_locals_valueArgAt_(m, locals, 0);
    
    if (other != self) 
    { 
	return ((IoState *)IOSTATE)->lobby; 
    }
    return self;
}

IoObject *IoNil_isNil(IoObject *self, IoObject *locals, IoMessage *m)
{ 
    /*#io
    docSlot("isNil", 
    "Returns Nop.")
    */
    
    return ((IoState *)(IOSTATE))->lobby; 
}

IoObject *IoNil_ifNil(IoObject *self, IoObject *locals, IoMessage *m)
{ 
    /*#io
    docSlot("ifNil(<message>)", 
    "Evaluates message.")
    */

    return IoMessage_locals_valueArgAt_(m, locals, 0); 
}

IoObject *IoNil_ifTrue(IoObject *self, IoObject *locals, IoMessage *m)
{ 
    /*
    undocumented: yes
    docSlot("ifTrue(<message>)", 
    "Returns Nop.")
    */
    
    return ((IoState *)(IOSTATE))->lobby; 
}

IoObject *IoNil_ifFalse(IoObject *self, IoObject *locals, IoMessage *m)
{ 
    /*
    undocumented: yes
    docSlot("ifFalse(<message>)", 
    "Evaluates message.")
    */
    
    return IoMessage_locals_valueArgAt_(m, locals, 0); 
}

