/*
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
*/

#include "IoState.h"
#include "IoObject.h"
#include "IoObject_actor.h"
#include "IoNil.h"
#include "IoSeq.h"
#include "IoException.h"

void IoState_error_description_(IoState *self, 
    IoMessage *m, 
    const char *name, 
    const char *format, 
    ...)
{
    IoSymbol *description;
    ByteArray *ba;
    va_list ap;
    va_start(ap, format);
    ba = ByteArray_newWithVargs_(format, ap);
    description = IoState_symbolWithByteArray_copy_(self, ba, 0);
    va_end(ap);

    if (IoState_garbageCollectorIsPaused(self))
    { 
	fputs(CSTRING(description), stderr); 
    }

    IoState_raiseExceptionWithName_andDescription_(
	self, SIOSYMBOL((char *)name), description, m);
}


/* --- exception handling --- */

void IoState_syncRetainStack(IoState *self)
{ 
    self->currentRetainStack = self->currentActor->coro->retainedValues; 
}

/* --- called by IoObject_error --- */

void IoState_raiseExceptionWithName_andDescription_(IoState *self, 
  IoSymbol *name, 
  IoSymbol *description, 
  IoMessage *m)
{
    IoException *e = IoException_new(self);
    IoException_name_(e, name);
    IoException_description_(e, description);
    
    if (m) 
    {
	IoException_caughtMessage_(e, m);
    }
    
    IoException_justRaise(e);
}

IoObject *IoState_withinCatchPerform(IoState *self, 
  IoObject *target, 
  IoObject *locals, 
  IoMessage *m)
{
    IoException *e;
    IoObject *r = self->ioNil;
    r = IoState_tryFunc_(self, (IoCatchCallback *)IoObject_perform, target, locals, m, &e);
    
    if (e) 
    {
	IoState_exception_(self, e);
    }
    
    return r;
}

IoObject *IoState_tryFunc_(IoState *self, 
  IoCatchCallback *func, 
  void *arg1,
  void *arg2,
  void *arg3, 
  IoException **e)
{
    IoCatch *c = IoCatch_new(self);
    IoObject *r = IoCatch_tryFunc_(c, (IoCatchCallback *)func, arg1, arg2, arg3, e);
    IoCatch_free(c);
    return r;
}


