
Object do(
    docsSlot := method(
	self hasSlot("docs") ifNil(self docs := Object clone do(slots := Object clone))
	self docs
    )
    
    docSlot := method(k, v, 
	entry := Object clone 
	entry description := v asSymbol
	if (k containsSeq("("), 
	    entry args := k afterSeq("(") beforeSeq(")") split(",") map(v, v strip)
	)	
	self docsSlot slots setSlot(k beforeSeq("("), entry)

	//self docsSlot slots setSlot(k beforeSeq("("), v)
	//writeln("docSlot(", k, ", ", self docsSlot slots getSlot(k), ")")
    )
    
    docCopyright := method(v, 
	k := thisMessage name asMutable removePrefix("doc") makeFirstCharacterLowerCase
	self docsSlot setSlot(k, v)
    )
    
    docLicense := getSlot("docCopyright")
    docObject := getSlot("docCopyright")
    docDescription := getSlot("docCopyright")
    docCredits := getSlot("docCopyright")
    docInclude := getSlot("docCopyright")
    docDependsOn := getSlot("docCopyright")

    ioDoc := method(
	//docsSlot
	if(thisMessage argAt(0), self doMessage(thisMessage argAt(0)))
    )


    
    docSlot("eval(...)", "Performs the argument (in the same manner as do() ) and returns the result. ")
    eval := method(self doMessage(thisMessage argAt(0)))
    
    docSlot("list(...)", "Returns a List containing the arguments.")
    list := method(thisMessage argsEvaluatedIn(sender))
    
    docSlot("..(arg)", ".. is an alias for: method(arg, self asString append(arg asString))")
    setSlot("..", method(arg, self asString appendSeq(arg asString)))
    
    docSlot("newSlot(slotName, aValue)", 
    """Creates a getter and setter for the slot with the name slotName 
    and sets it's default the value aValue. Returns self. For example, 
    newSlot("foo", 1) would create slot named foo with the value 1 as well as a setter method setFoo().""")

    newSlot := method(name, value, 
	self setSlot(name, value)
	self setSlot("set" .. name asCapitalized, 
		doString("method(" .. name .. " = sender doMessage(thisMessage argAt(0)); self)"))
	value
    )
    
    docSlot("launchFile(pathString)", 
    "Eval file at pathString as if from the command line in it's folder.")
    
    launchFile := method(path, args,
	args ifNil(args = List clone)
	Lobby args := args
	Lobby launchPath :=  path pathComponent
	Directory setCurrentWorkingDirectory(Lobby launchPath)
	self doFile(path)
    )
    
    docSlot("println", "Same as print, but also prints a new line. Returns self.")
    println := method(self print; write("\n"); self)





    docSlot("?(aMessage)", """
    description: Sends the message aMessage to the receiver if it can respond to it. Example:
    <pre>
    MyObject test // performs test
    MyObject ?test // performs test if MyObject has a slot named test
    </pre>
    The search for the slot only follows the receivers proto chain. 
    """)

    setSlot("?", method(
	m := thisMessage argAt(0)
	if (self getSlot(m name), self doMessage(m, sender), Nil)
    ))

    docSlot("ancestors", 
    "Returns a list of all of the receiver's ancestors as found by recursively following the protos links.") 

  ancestors := method(a, 
	if(a, if(a contains(self), return a), a = List clone) 
	a append(self)
	self protos foreach(p, p ancestors(a))
	a
  )
    
    docSlot("super(aMessage)", 
    """Sends the message aMessage to the receiver's proto with the context of self. Example:
    <pre>
    self test(1, 2)   // performs test(1, 2) on self
    super(test(1, 2)) // performs test(1, 2) on self proto but with the context of self
    </pre>
    """)

  setSlot("super", method(
	senderSlotContext := sender getSlot("slotContext")
	m := thisMessage argAt(0)
	m ifNil(Exception raise("Object", "super requires an argument"))
	senderSlotContext ifNil(Exception raise("Object", "super called outside of block context"))
	slotName := m name
	ancestor := senderSlotContext ancestorWithSlot(slotName)
	if(ancestor == Nil or ancestor uniqueId == senderSlotContext uniqueId, Exception raise("Object", "super slot " .. slotName .. " not found"))
	ancestor getSlot(slotName) performOn(sender getSlot("self"), sender, m, ancestor)
))

    docSlot("resend", 
    """Send the message used to activate the current method to the Object's proto.
    For example;
    <pre>
    Dog := Mammal clone do(
	init := method(
	    resend
	)
    )
    </pre>
    calling Dog init will send an init method to Mammal, but using the Dog's context.
    """)

  setSlot("resend", method(
	senderSlotContext := sender getSlot("slotContext")
	senderSlotContext ifNil(Exception raise("Object", "resend called outside of block context"))
	m := sender thisMessage
	slotName := m name
	ancestor := senderSlotContext ancestorWithSlot(slotName)
	if(ancestor == Nil or ancestor uniqueId == senderSlotContext uniqueId, Exception raise("Object", "resend slot " .. slotName .. " not found"))
	ancestor getSlot(slotName) performOn(sender getSlot("self"), sender, m, ancestor)
))
  do := method(
	self appendProto(sender)
	self doMessage(thisMessage argAt(0), target)
	self removeProto(sender)
	self
  )
    
    docSlot("list(...)", "Returns a List containing the arguments.")
    list := method(thisMessage argsEvaluatedIn(sender))
    
    docSlot("..(arg)", " .. is an alias for: method(arg, self asString append(arg asString))")
    setSlot("..", method(arg, self asString appendSeq(arg asString)))

    docSlot("newSlot(slotName, aValue)", """
    description: Creates a getter and setter for the slot with the name slotName 
    and sets it's default the value aValue. Returns self. For example, 
    newSlot("foo", 1) would create slot named foo with the value 1 as well as a setter method setFoo().
    """)

    newSlot := method(name, value, self setSlot(name, value); self setSlot("set" .. name asCapitalized, doString("method(" .. name .. " = sender doMessage(thisMessage argAt(0)); self)")); value)
    
    docSlot("println", "Same as print, but also prints a new line. Returns self.")
    println := method(self print; write("\n"); self)

    docSlot("in(aList)", "Same as: aList contains(self)")
    in := method(aList, aList contains(self)) 
)