#!/usr/bin/env io

Message do (
    isOperator := method(
        arguments size == 0 and ops hasKey(name)
    )

    operatorPrecedence := method(
        arguments size == 0 or return Nil
        ops at(name)
    )
)


ops := Map clone do (
    op := getSlot("atPut")

    op("'",   0)
    op(".",   0)
    op("?",   0)
    op("(",   0)
    op(")",   0)

    op("^",   1)

    op("++",  2)
    op("--",  2)

    op("*",   3)
    op("/",   3)
    op("%",   3)

    op("+",   4)
    op("-",   4)

    op("<<",  5)
    op(">>",  5)

    op(">",   6)
    op("<",   6)
    op("<=",  6)
    op(">=",  6)

    op("==",  7)
    op("!=",  7)

    op("&",   8)

    op("|",   9)

    op("and", 10)
    op("&&",  10)

    op("or",  11)
    op("||",  11)

    op("..",  12)

    op("=",   13)
    op("+=",  13)
    op("-=",  13)
    op("*=",  13)
    op("/=",  13)
    op("%=",  13)
    op("&=",  13)
    op("^=",  13)
    op("|=",  13)
    op("<<=", 13)
    op(">>=", 13)
    op(":=",  13)
    op("<-",  13)
    op("<->", 13)
    op("->",  13)

    op("return",  14)

    op(",", 15)
    op(";", 15)

    maxLevel := 16

    removeSlot("op")
)

List popFirst := method(
    item := first
    if(item, removeAt(0))
    item
)

Message do( 
    addArg := method(m,
	self setArguments(self arguments append(m))
    )

    parse := method(tokens,
        tokens first isMessage or return
	//writeln("parse ", tokens first name)
    	m := Message clone
	m parseName(tokens)
	m parseArgs(tokens)
	m parseAttached(tokens)
	while(m parseNext(tokens), Nop)
	m
    )

    parseName := method(tokens,
        //writeln("parseName ", tokens first name, " ", tokens first type)
	token := tokens popFirst
	setName(token name)
        token isLiteral and setCachedResult(doString(token name))
    )
    
    parseArgs := method(tokens,
	if (tokens first type != "OpenParen", return)
	tokens popFirst
	//writeln("parseArgs ", tokens first name)
	
	while(arg := parse(tokens), 
	    addArg(arg)
	    if (tokens first type != "Comma", break)
	    tokens popFirst
	)
	
	if (tokens first type != "CloseParen", Exception raise("Message", "missing close paren"))
	tokens popFirst
    )
    
    
    parseAttached := method(tokens,
        tokens first ?isMessage or return
	//writeln("parseAttached ", tokens first name)
	m := parse(tokens)
	if (m, 
	    setAttachedMessage(m)
	)
    )
    
    parseNext := method(tokens,
	if (tokens first type == "Terminator",
	    tokens popFirst
	    m := parse(tokens)
	    setNextMessage(m)
	    m
	)
	Nil
    )
)

Token := Object clone do(
    isMessage := method(
	t := self type
	t == "Identifier" or t == "Operator" or t == "MonoQuote" or t == "TriQuote" or t == "Number" or t == "HexNumber"
    )
    isLiteral := method(
	t := self type
	t == "MonoQuote" or t == "TriQuote" or t == "Number" or t == "HexNumber"
    )
)

Object do (
    stackLoop := block(
        stackName := thisMessage argAt(0) name
        indexName := thisMessage argAt(1) name
        stack := list(sender doMessage(thisMessage argAt(2)))
        body := thisMessage argAt(3)

        sender setSlot(stackName, stack)
        while(stack size > 0,
            sender setSlot(indexName, stack pop)
            sender doMessage(body)
        )
    )
)

Tail := Object clone do (
    newSlot("msg")
    newSlot("level", -1)

    attach := method(m,
        pushImpl(m)
        AttachTail clone setMsg(m) setLevel(level)
    )

    setNext := method(m,
        msg setNextMessage(m)
        AttachTail clone setMsg(m) setLevel(level)
    )

    finish := method(
        msg setAttachedMessage
        self
    )

    code := method(msg code)
)

AttachTail := Tail clone do (pushImpl := method(m, msg setAttachedMessage(m)))
FirstArgTail := Tail clone do (pushImpl := method(m, msg setArguments(list(m))))
NewTail := Tail clone do (pushImpl := method(m, msg = m))

MessageStack := Object clone do (
    first := Nil
    stack := Nil
    last := Nil

    init := method(
        first = NewTail clone setLevel(ops maxLevel)
        stack = list(first)
    )

    attach := method(msg,
        last = msg
        if (msg isOperator,
            level := msg operatorPrecedence
            while (stack last and stack last level <= level,
                stack pop finish
            )
            stack push(stack pop attach(msg))
            stack push(FirstArgTail clone setMsg(msg) setLevel(level))
        ,
            stack push(stack pop attach(msg))
        )
        #dumpStack
    )

    nextMessage := method(
        m := last nextMessage
        tail := Nil; while (stack last, tail = stack pop finish)
        tail setNext(m)
        if (tail msg != last,
            last setNextMessage
        )
        self init
        m
    )

    dumpStack := method(
        sep := block(sender sep = block(write(", ")))
        stack foreach(v,
            sep
            write(v msg ?name, ": ", v level)
        )
        writeln
    )

    code := method(first code)
)

Compiler do(
    parse := method(text, 
	tokens := tokensForString(text)
	tokens foreach(t, t appendProto(Token))
	tokens parens := List clone
	m := Message parse(tokens)
        post := postProcess(m clone)

	writeln("parsing: ", text)
	writeln("parsed:  ", m code asMutable replaceSeq("\n", " "))
        writeln("post:    ", post code asMutable replaceSeq("\n", " "))
        writeln
    )

    postProcess := method(msg,
        stackLoop(argsToProcess, m, msg,
            stack := MessageStack clone

            while (m,
                while (m,
                    stack attach(m)

                    argsToProcess appendSeq(m arguments)
                    m = m attachedMessage
                )

                m = stack nextMessage
            )
        )

        msg
    )
)

Compiler parse("a := 1")
Compiler parse("a b(c, d); e(f); g h")
Compiler parse("\"a\" append(\"b\")")
Compiler parse("0x1c + 19")
Compiler parse("a = f(a + b, c * d); b = 19")
Compiler parse("a + b * c + d; e + f")
Compiler parse("a = 1; b = 2; c = 3")
Compiler parse("a = 1; b = 2; c = 3")
Compiler parse("a; b; c; d")
Compiler parse("a(1) (b; c; d) e")


# message(a) setAttachedMessage(message(=) setAttachedMessage(message(1)))
# message(updateSlot) setArguments(list(message(a) name, message(1)))
# 
# x; a b =() c d; y
# x; a updateSlot("b", c d); y
# 
# x; b =() c d; y
# x; updateSlot("b", c d); y

#+ x y z + a b c
#+(x y z) +(a b c)
#
#a + b * c * d + e
#a +(b *(c) *(d)) +(e)
#
#a + b * c ^ d; e
#a +(b *(c ^(d))); e

#if (name == "=", Nop
#    writeln(Message clone setName("updateSlot") setArguments(
#        list(
#            Message clone setName("\"" .. m attachedTo name .. "\"") setCachedResult(m attachedTo name),
#            m attachedMessage
#        )
#    ) code asMutable replaceSeq("\n", " "))
#)

#Object setSlot("'", getSlot("message"))
#Object setSlot("squareBrackets", getSlot("message"))
#
#f := MessageStack clone
#list(
#    'a, 'b, [+], 'c, 'd, [*], 'e, 'f, [*], 'g, [+], 'h
##    F   F   4    4   4   3    3   3   3    3   4    4
#) foreach(v, f append(v))
#
#
#writeln
#f stack foreach(i, tail,
#    writeln(i, ": ", if (tail level == -1, "-", tail level), " ", tail msg name)
#)
#writeln
#writeln(f code)
