/*   
    Copyright (c) 2005, Steve Dekorte
    All rights reserved. See _License.txt.
*/

#include <stdio.h>
#include <stdlib.h>

int main(int argc, const char *argv[])
{
    if (argc != 4)
    {
	printf("usage: io2c [ObjectName] [inFile] [outFile]\n\n");
    }
    else
    {
	const char *objectName  = argv[1];
	const char *fileName    = argv[2];
	const char *outFileName = argv[3];

	FILE *in  = fopen(fileName, "r");
	FILE *out = fopen(outFileName, "w");
	
	if (!in)
	{
	    printf("unable to open input file %s\n", fileName);
	    exit(-1);
	}
	
	if (!out)
	{
	    printf("unable to open output file %s\n", outFileName);
	    exit(-1);
	}
	
	fputs("#include \"IoState.h\"\n", out);
	fputs("#include \"IoObject.h\"\n\n", out);
	fputs("void Io", out);
	fputs(objectName, out);
	fputs("Init(IoState *self, IoObject *context)\n{\n", out);
	fputs("  char *s = ", out);

	fputc('"', out);
	
	for (;;)
	{
	    char c = fgetc(in);
	    
	    if (feof(in)) break;
	    
	    if (c == '\\' || c == '"') 
	    {
		fputc('\\', out);
	    }
	    
	    if (c == '\n') 
	    {
		fputs("\\n\"\n  ", out);
		fputc('"', out);
	    }
	    else if (c == '\r') 
	    {
		fputs("\\r\"\r  ", out);
		fputc('"', out);
	    }
	    else
	    {
		fputc(c, out);
	    }
	}

	fputs("\";\n\n", out);
	fprintf(out, "  IoObject_rawDoString_label_(context, SIOSYMBOL(s), SIOSYMBOL(\"%s\"));\n", objectName);
	fputs("}\n", out);
	
	fclose(in);
	fclose(out);

	printf("made %s from %s\n", outFileName, fileName);
    }
    return 0;
}
