
Block ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Block")
docDescription("Blocks are anonymous functions (messages with their own locals object). They are typically used to represent object methods.")


    docSlot("print", 
    "prints an Io source code representation of the block/method")
    

    docSlot("code", 
    "Returns a string containing the decompiled code of the receiver. ")
    

    docSlot("setCode(aString)", 
    "Set's the reciever's message to a compiled version of aString. Returns self")
    

    docSlot("message", 
    "Returns the root message of the receiver. ")
    

    docSlot("setMessage(aMessage)", 
    "Sets the root message of the receiver to aMessage. ")
    

    docSlot("argumentNames", 
    "Returns a List of strings containing the argument names of the receiver. ")
    

    docSlot("setArgumentNames(aListOfStrings)", 
    "Sets the receiver's argument names to those specified in 
    aListOfStrings. Returns self.  ")
    

    docSlot("scope", 
    "Returns the scope used when the block is activated or 
    Nil if the target of the message is the scope.   ")
    

    docSlot("setScope(anObjectOrNil)", 
    "If argument is an object, when the block is activated, 
    it will set the proto and self slots of it's locals to the specified 
    object. If Nil, it will set them to the target of the message. ")
    

    docSlot("performOn(anObject)", 
    "Activates the receiver in the target context of anObject. 
    Returns the result.")
    
)

CFunction ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("CFunction")
docDescription("A container for a pointer to a C function binding. CFunction's can only be defined from the C side and act like blocks in that when placed in a slot, are called when the slot is activated. The for, if, while and clone methods of the Lobby are examples of CFunctions. CFunctions are useful for implementing methods that require the speed of C or binding to a C library.")


    docSlot("id", 
    "Returns a number containing a unique id for the receiver's internal C function. ")
    

    docSlot("==(anObject)", 
    "Returns self if the argument is a CFunction with the 
    same internal C function pointer. ")
    
)

Collector ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Collector")
docDescription("""Contains methods related to Io's garbage collector. Io currently uses a incremental, non-moving, generational collector based on the tri-color (black/gray/white) algorithm with a write-barrier. 
<p>
Every N number of object allocs, the collector will walk some of the objects marked as gray, marking their connected white objects as gray and turning themselves black. Every M allocs, it will pause for a sweep where it makes sure all grays are marked black and frees all whites. 
<p>
If the sweepsPerGeneration is set to zero, it will immediately mark all blacks as white again and mark the root objects as gray. Otherwise, it will wait until the sweepsPerGeneration count is reached to do this. By adjusting the allocsPerSweep and sweepsPerGeneration appropriately, the collector can be tuned efficiently for various usage cases. Generally, the more objects in your heap, the larger you'll want this number.""")


    docSlot("collect", "Runs garbage collector. Returns the number of items collected. ")
    

    docSlot("debugOn", "Turn on printing of collector debugging messages. Returns self.")
    

    docSlot("debugOff", "Turn off printing of collector debugging messages. Returns self.")
    

    docSlot("setAllocsPerSweep(aNumber)", "Sets the number of object allocations per sweep. Returns self.")
    

    docSlot("allocsPerSweep", 
    "Return the number of allocations per collector sweep.")
    

    docSlot("setSweepsPerGeneration(aNumber)", 
    "Sets the number of sweeps before black marks are turned to white. Returns self.")
    

    docSlot("sweepsPerGeneration", "Return the number of sweeps per generation.")
    
)

Compiler ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Compiler")
docDescription("Contains methods related to the compiling code.")


    docSlot("messageForTokens(aList)", 
    "Returns the compiled message object for the given token list.")
    

    docSlot("messageForString(aString, optionalLabelString)", 
    "Returns the compiled message object for aString.")
    
)

Date ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Date")
docDescription("A container for a date and time information.
credits: fromString method by Sean Perry")


    docSlot("now", 
    "Sets the receiver to the current time. Returns self.")
    

    docSlot("clock", 
    "Returns a number containing the number of seconds 
    of processor time since the beginning of the program or -1 if unavailable. ")
    

    docSlot("cpuSecondsToRun(<message>)", 
    "Evaluates message and returns a Number whose value is the cpu seconds taken to do the evaluation.")
    

    docSlot("year", 
    "Returns a number containing the year of the receiver. ")
    

    docSlot("setYear(aNumber)", 
    "Sets the year of the receiver. ")
    

    docSlot("month", 
    "Returns a number containing the month(1-12) of the year of the receiver. ")
    

    docSlot("setMonth(aNumber)", 
    "Sets the month(1-12) of the receiver. Returns self. ")
    

    docSlot("day", 
    "Returns a number containing the day of the month of the receiver. ")
    

    docSlot("setDay(aNumber)", 
    "Sets the day of the receiver. Returns self.")
    

    docSlot("hour", 
    "Returns a number containing the hour of the day(0-23) of the receiver. ")
    

    docSlot("setHour(aNumber)", 
    "Sets the hour of the receiver. Returns self.")
    

    docSlot("minute", 
    "Returns a number containing the minute of the hour(0-59) of the receiver. ")
    

    docSlot("setMinute(aNumber)", 
    "Sets the minute of the receiver. Returns self.")
    

    docSlot("second", 
    "Returns a number containing the seconds of the minute(0-59) of the receiver. This number may contain fractions of seconds. ")
    

    docSlot("setSecond(aNumber)", 
    "Sets the second of the receiver. Returns self.")
    

    docSlot("zone", 
    "Returns a string containing the time zone code of the receiver. ")
    

    docSlot("isDaylightSavingsTime", 
    "Returns self if Daylight Saving Time is in effect for the receiver, otherwise returns Nil. ")
    

    docSlot("validTime(hour, min, sec)", 
    "Returns self if the specified time is valid, otherwise returns Nil. A negative value will count back; i.e., a value of -5 for the hour, will count back 5 hours to return a value of 19. No adjustment is done for values above 24.")
    

    docSlot("secondsSince(aDate)", 
    "Returns a number of seconds of between aDate and the receiver. ")
    

    docSlot("secondsToNow(aDate)", "Returns the number of seconds since aDate. ")
    

    docSlot("dateAfterSeconds(secondsNumber)", 
    "Returns a new date that is secondsNumber seconds after the receiver. ")
    

    docSlot("asString(optionalFormatString)", 
    """Returns a string representation of the receiver using the 
    receivers format. If the optionalFormatString argument is present, the 
    receiver's format is set to it first. Formatting is according to ANSI C 
    date formating rules.

    <pre>
    %a abbreviated weekday name (Sun, Mon, etc.)
    %A full weekday name (Sunday, Monday, etc.) 
    %b abbreviated month name (Jan, Feb, etc.) 
    %B full month name (January, February, etc.) 
    %c full date and time string 
    %d day of the month as two-digit decimal integer (01-31) 
    %H hour as two-digit 24-hour clock decimal integer (00-23) 
    %I hour as two-digit 12-hour clock decimal integer (01-12) 
    %m month as a two-digit decimal integer (01-12)
    %M minute as a two-digit decimal integer (00-59) 
    %p either "AM" or "PM" 
    %S second as a two-digit decimal integer (00-59) 
    %U number of week in the year as two-digit decimal integer (00-52) 
       with Sunday considered as first day of the week 
    %w weekday as one-digit decimal integer (0-6) with Sunday as 0 
    %W number of week in the year as two-digit decimal integer (00-52) 
       with Monday considered as first day of the week 
    %x full date string (no time); in the C locale, this is equivalent 
       to "%m/%d/%y". 
    %y year without century as two-digit decimal number (00-99) 
    %Y year with century as four-digit decimal number 
    %Z time zone name (e.g. EST); 
       null string if no time zone can be obtained 
    %% stands for '%' character in output string. 
    </pre>
    """)
    

    docSlot("print", "Prints the receiver. Returns self.")
    

    docSlot("asNumber", "Returns the date as seconds since 1970.")
    

    docSlot("fromNumber(aNumber)", "Sets the receiver to be aNumber seconds since 1970.")
    

    docSlot("fromString(aString, formatString)", "Sets the receiver to the date specified by aString as parsed according to the given formatString. See the Date asString method for formating rules. Returns self. ")
    

    docSlot("-(aDurationOrDate)", "Return a new Date with the receiver's value minus an amount of time specified by aDuration to the receiver. Returns self. ")
    

    docSlot("-=(aDuration)", "Subtract aDuration from the receiver. Returns self.") 
    

    docSlot("+=(aDuration)", "Add aDuration to the receiver. Returns self. ")
    

    docSlot("+(aDuration)", "Return a new Date with the receiver's value plus an amount of time specified by aDuration object to the receiver. ")
    
)

Debugger ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Debugger")
docDescription("Contains methods related to the IoVM debugger.")


    docSlot("vmWillSendMessage", 
    "When debugging for one or more actors is turned on 
    (by sending an actor a turnOnMessageDebugging message), a 
    vmWillSendMessage message will be sent to the Debugger object 
    for each message send within the coroutines of the actors being 
    debugged. The first argument will contain the actor and the 
    second argument will contain the message that is about to be 
    sent. You will need to send a resume message to that actor in 
    order for it to continue execution.")

)

Directory ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Directory")
docDescription("""The Directory object supports accessing filesystem directories. A note on paths; if a path begins with a "/" it's the root, if it beings with a "./" it's the launch path, if not specified, "./" is assumed.


credits: Cygwin code by Mike Austin. WIN32 code by Daniel Vollmer.""")


    docSlot("path", 
    """Returns the directory path. The default path is ".". """)
    

    docSlot("setPath(aString)", 
    "Sets the directory path. Returns self. ")
    

    docSlot("name", 
    "Returns the receiver's last path component.  ")
    

    docSlot("exists(optionalPath)", 
    "Returns self if the Directory path exists, and Nil otherwise. 
    If optionalPath string is provided, it tests the existance of that path instead. ")
    

    docSlot("items", 
    "Returns a list object containing File and Directory objects 
    for the files and directories of the receiver's path. ")
    

    docSlot("at(aString)", 
    "Returns a File or Directory object matching the name specified 
    by aString or Nil if no such file or directory exists. ")
    

    docSlot("createSubdirectory(name)", 
    "Create a subdirectory with the specified name.")
    

    docSlot("create", 
    "Create the directory if it doesn't exist.")
    

    docSlot("size", 
    "Returns a Number containing the number of file and directory 
    object at the receiver's path. ")
    

    docSlot("currentWorkingDirectory", 
    "Returns the current working directory path.")
    

    docSlot("setCurrentWorkingDirectory(pathString)", 
    "Set's the current working directory path. 
    Returns self on success or Nil on error.")
    
)

Duration ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Duration")
docDescription("A container for a duration of time.")


    docSlot("years", 
    "Returns a number containing the year of the receiver. ")
    

    docSlot("setYears(aNumber)", 
    "Sets the year of the receiver. Returns self.")
    

    docSlot("days", 
    "Returns a number containing the day of the month of the receiver. ")
    

    docSlot("setDays(aNumber)", 
    "Sets the day of the receiver. Returns self.")
    

    docSlot("hours", 
    "Returns a number containing the hour of the day(0-23) of the receiver. ")
    

    docSlot("setHours(aNumber)", 
    "Sets the hour of the receiver. Returns self.")
    

    docSlot("minutes", 
    "Returns a number containing the minute of the hour(0-59) of the receiver. ")
    

    docSlot("setMinutes(aNumber)", 
    "Sets the minute of the receiver. Returns self.")
    

    docSlot("seconds", 
    "Returns a number containing the seconds of the minute(0-59) of the receiver. 
    This number may contain fractions of seconds. ")
    

    docSlot("setSeconds(aNumber)", 
    "Sets the second of the receiver. Return self.")
    

    docSlot("asString(formatString)", 
    """Returns a string representation of the receiver. The formatString argument is optional. If present, the returned string will be formatted according to ANSI C date formating rules.

    <pre>
    %a abbreviated weekday name (Sun, Mon, etc.)
    %A full weekday name (Sunday, Monday, etc.) 
    %b abbreviated month name (Jan, Feb, etc.) 
    %B full month name (January, February, etc.) 
    %c full date and time string 
    %d day of the month as two-digit decimal integer (01-31) 
    %H hour as two-digit 24-hour clock decimal integer (00-23) 
    %I hour as two-digit 12-hour clock decimal integer (01-12) 
    %m month as a two-digit decimal integer (01-12)
    %M minute as a two-digit decimal integer (00-59) 
    %p either "AM" or "PM" 
    %S second as a two-digit decimal integer (00-59) 
    %U number of week in the year as two-digit decimal integer (00-52) 
       with Sunday considered as first day of the week 
       
    %w weekday as one-digit decimal integer (0-6) with Sunday as 0 
    %W number of week in the year as two-digit decimal integer (00-52) 
       with Monday considered as first day of the week 
       
    %y year without century as two-digit decimal number (00-99) 
    %Y year with century as four-digit decimal number 
    %Z time zone name (e.g. EST); null string if no time zone can be obtained 
    %% stands for '%' character in output string. 

    %x full date string (no time); in the C locale, 
       this is equivalent to "%%m/%%d/%%y". 
       
    %X full time string (no date); in the C locale, 
       this is equivalent to "%%H:%%M:%%S".</pre>

    The default format is "%Y-%m-%d %H:%M:%S %Z". 
    """)
    

    docSlot("print", "Prints the receiver. Returns self.")
    

    docSlot("asNumber", 
    "Returns a number representation of the receiver. 
    (where 1 is equal to one second) ")
    

    docSlot("fromNumber(aNumber)", 
    "Sets the receiver to the Duration specified by 
    aNumber(same format number as returned by Duration asNumber). Returns self. ")
    

    docSlot("+=(aDuration)", "Add aDuration to the receiver. Returns self. ")
    

    docSlot("-=(aDuration)", "Subtract aDuration to the receiver. Returns self. ")
    
)

DynLib ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("DynLib")
docDescription("A DLL Loader by Kentaro A. Kurahone <kurahone@sigusr1.org>.")


    docSlot("setPath(aString)", 
    "Sets the path to the dynamic library. Returns self.")
    

    docSlot("path", 
    "Returns the path to the dynamic library.")
    

    docSlot("setInitFuncName(aString)", 
    "Sets the initialization function name for the dynamic library. Returns self.")
    

    docSlot("initFuncName", 
    "Returns the initialization function name.")
    

    docSlot("setFreeFuncName(aString)", 
    "Sets the free function name. Returns self.")
    

    docSlot("freeFuncName", 
    "Returns the free function name.")
    

    docSlot("open", 
    "Opens the dynamic library and returns self or raises a DynLoad.open Error if there is an error. ")
    

    docSlot("close", 
    "Closes the library. Returns self.")
    

    docSlot("isOpen", 
    "Returns self if the library is open, or Nil otherwise.")
    
)

Exception ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Exception")
docDescription("""The Exception proto is used to raise and catch exceptions. 
<p>
Example of raising an Exception:
<pre>Exception raise("some name", "some description")
</pre>

Example of catching an Exception:
<pre>try(
  // code
) catch(Warning, e, 
  // code
) catch(Error, e, 
  // code
)
</pre>
""")


    docSlot("raise(name, description)", 
    "Raise a new, unresumable exception and set it's name and description slots. ")
    

    docSlot("raiseResumable(name, description)", 
    "Same as raise() except the exception will be resumable. ")
    

    docSlot("catch(exceptionProto, localSlot, doMessage)", 
    "Sets the localSlot to the receiver and performs the doMessage 
    if the receiver is the exceptionProto or has exceptionProto as a parent. ")
    

    docSlot("pass", 
    "Pass the exception up to the next try().")
    

    docSlot("resume", 
    "Continue executing at the point where the exception 
    was raised (if the exception is resumable). ")
    

    docSlot("isResumable", 
    "Returns self if the reciever is resumable, Nil otherwise. ")
    

    docSlot("backTraceString", 
    "Returns a String containing a description of the back trace.")
    
)

File ioDoc(
 docCopyright("Steve Dekorte", 2002)
 docLicense("BSD revised") 
 docObject("File")  
 docDescription("""Encapsulates file i/o. Here's an example of opening a file, and reversing it's lines:
 
 <pre>
 file = File clone openForUpdating("/tmp/test")
 lines = file readLines reverse
 file rewind
 lines foreach(i, line, file write(line, "\n"))
 file close</pre>
 """)
 

    docSlot("standardInput", 
    "Returns a new File whose stream is set to the standard input stream.")
    

    docSlot("standardOutput", 
    "Returns a new File whose stream is set to the standard output stream.")
    

    docSlot("standardError", 
    "Returns a new File whose stream is set to the standard error stream.")
    

    docSlot("setPath(aString)", 
    "Sets the file path of the receiver to pathString. 
    The default path is an empty string. Returns self.")
    

    docSlot("path", 
    "Returns the file path of the receiver.")
    

    docSlot("name", 
    "Returns the last path component of the file path.")
    

    docSlot("mode", 
    "Returns the open mode of the file(either read, update or append).")
    

    docSlot("temporaryFile", 
    "Returns a new File object with an open temporary file. The file is 
    automatically deleted when the returned File object is closed or garbage collected. ")
    

    docSlot("openForReading(optionalPathString)", 
    "Sets the file mode to read (reading only) and calls open(optionalPathString). ")
    

    docSlot("openForUpdating(optionalPathString)", 
    "Sets the file mode to update (reading and writing) and calls 
    open(optionalPathString). This will not delete the file if it already exists. 
    Use the remove method first if you need to delete an existing file before opening a new one.")
    

    docSlot("openForAppending(optionalPathString)", 
    "Sets the file mode to append (writing to the end of the file) 
    and calls open(optionalPathString).")
    

    docSlot("open(optionalPathString)", 
    "Opens the file. Creates one if it does not exist. 
    If the optionalPathString argument is provided, the path is set to it before 
    opening. Returns self or raises an File exception on error.")
    

    docSlot("popen", 
    "open as a pipe")
    

    docSlot("close", 
    "Closes the receiver if open, otherwise does nothing. Returns self.")
    

    docSlot("flush", 
    "Forces any buffered data to be written to disk. Returns self.")
    

    docSlot("asString", 
    "Returns self.")
    

    docSlot("asBuffer", 
    "Opens the receiver in read only mode, reads the whole 
    contents of the file into a buffer object, closes the file and returns the buffer.")
    

    docSlot("exists", 
    "Returns self if the file specified by the receiver's 
    path exists, Nil otherwise.")
    

    docSlot("remove", 
    "Removes the file specified by the receiver's path. 
    Raises an File doesNotExist exception if the file does not exist. Returns self.")
    

    docSlot("truncateToSize(numberOfBytes)", 
    "Trunctates the file's size to the numberOfBytes. Returns self.")
    

    docSlot("moveTo(pathString)", 
    """Moves the file specified by the receiver's path to the 
    new path pathString. Raises an File doesNotExist exception if the 
    file does not exist or a File nameConflict exception if the file 
    nameString already exists.""")
    

    docSlot("write(aSequence1, aSequence2, ...)", 
    "Writes the arguments to the receiver file. Returns self.")
    

    docSlot("readLines", 
    "Returns list containing all lines in the file.")
    

    docSlot("readLine", 
    "Reads the next line of the file and returns it as a 
    string without the return character. Returns Nil if the end of the file has been reached.")
    

    docSlot("readToBufferOfLength(aBuffer, aNumber)", 
    "Reads the aNumber bytes and appends them to aBuffer.
    Returns the number of bytes read.")
    

    docSlot("readBufferOfLength(aNumber)", 
    "Reads a Buffer of the specified length and returns it.
    Returns Nil if the end of the file has been reached.")
    

    docSlot("readStringOfLength(aNumber)", 
    "Reads a String of the specified length and returns it.
    Returns Nil if the end of the file has been reached.")
    

    docSlot("rewind", 
    "Sets the file position pointer to the beginning of the file.")
    

    docSlot("setPosition(aNumber)", 
    "Sets the file position pointer to the byte specified by aNumber. Returns self.")
    

    docSlot("position", 
    "Returns the current file pointer byte position as a Number.")
    

    docSlot("isAtEnd", 
    "Returns self if the file is at it's end. Otherwise returns Nil.")
    

    docSlot("size", 
    "Returns the file size in bytes.")
    

    docSlot("isOpen", 
    "Returns self if the file is open. Otherwise returns Nil.")
    

    docSlot("at(aNumber)", 
    "Returns a Number containing the byte at the specified 
    byte index or Nil if the index is out of bounds.")
    

    docSlot("atPut(positionNumber, byteNumber)", 
    "Writes the byte value of byteNumber to the file position 
    positionNumber. Returns self.")
    

    docSlot("foreach(optionalIndex, value, message)", 
    """For each byte, set index to the index of the byte 
    and value to the number containing the byte value and execute aMessage.
    Example usage:
    <pre>
    aFile foreach(i, v, writeln("byte at ", i, " is ", v))
    aFile foreach(v, writeln("byte ", v))
    </pre>
    """)
    
)

File ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docCredits("Initial version contributed by Miles Egan.")


    docSlot("stat", 
    "Updates the receiver's meta info cache.")
    

    docSlot("protectionMode", 
    "Returns a Number containing the protection mode 
    associated with the file's path.")
    

    docSlot("lastAccessDate", 
    "Returns a Date object containing the last date and
    time the file was accessed.")
    

    docSlot("lastInfoChangeDate", 
    "Returns a Date object containing the last date and
    time the file's meta info was changed.")
    

    docSlot("lastDataChangeDate", 
    "Returns a Date object containing the last date and
    time the file's contents were changed.")
    

    docSlot("userId", 
    "Returns a Number containing the user id associated with the file's path.")
    

    docSlot("groupId", 
    "Returns a Number containing the group id associated with the file's path.")
    

    docSlot("statSize", 
    "Returns the file's size in bytes as a Number.")
    

    docSlot("isDirectory", 
    "Returns self if the receiver's path points to a directory, Nil otherwise.")
    

    docSlot("isPipe", 
    "Returns self if the receiver is a pipe, Nil otherwise.")
    

    docSlot("isLink", 
    "Returns self if the receiver's path points to a link, Nil otherwise.")
    

    docSlot("isRegularFile", 
    "Returns self if the receiver's file descriptor is a regular file, Nil otherwise.")
    

    docSlot("isSocket", 
    "Returns self if the receiver's file descriptor is a Socket, Nil otherwise.")
    

    docSlot("isUserExecutable", 
    "Returns self if the receiver is user group executable, Nil otherwise.")
    
)

Future ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")    
docObject("Future")        
docDescription("A transparent container for a value returned by an asynchronous message. The future effectively becomes the return value when it is ready. If the future is sent a message before the asynchronous message is processed, the current coroutine will pause until the result is ready.")


    docSlot("isWaitingFuture", 
    "Returns Nop. ")
    

    docSlot("forward", 
    "Implements Future's waiting behavior. ")
    
)

List ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("List")
docInclude("_ioCode/List.io")
docDescription("An mutable array of values. The first index is 0.")


    docSlot("with(anObject, ...)", 
    "Returns a new List containing the arguments. ")
    

    docSlot("indexOf(anObject)", 
    "Returns the index of the first occurrence of anObject 
    in the receiver. Returns Nil if the receiver doesn't contain anObject. ")
    

    docSlot("contains(anObject)", 
    "Returns self if the receiver contains anObject, otherwise returns Nil. ")
    

    docSlot("size", 
    "Returns the number of items in the receiver. ")
    

    docSlot("at(index)", 
    "Returns the value at index. Returns Nil if the index is out of bounds. ")
    

    docSlot("first(optionalSize)", 
    "Returns the first item or Nil if the list is empty. 
    If optionalSize is provided, that number of the first items in the list are returned. ")
    

    docSlot("last(optionalSize)", 
    "Returns the last item or Nil if the list is empty. 
    If optionalSize is provided, that number of the last items in the list are returned. ")
    

    docSlot("slice(startIndex, endIndex)", 
    "Returns a new string containing the subset of the 
    receiver from the startIndex to the endIndex. The endIndex argument 
    is optional. If not given, it is assumed to be the end of the string. ")
    

    docSlot("print", 
    "Prints the list. Returns self.")
    

    docSlot("foreach(optionalIndex, value, message)", 
    """Loops over the list values setting the specified index and 
    value slots and executing the message. Returns the result of the last 
    execution of the message. Example:
    <pre>list(1, 2, 3) foreach(i, v, writeln(i, " = ", v))
list(1, 2, 3) foreach(v, writeln(v))</pre>
    """)
    

    docSlot("reverseForeach(index, value, message)", 
    "Same as foreach, but in reverse order.")
    

    docSlot("select(optionalIndex, value, message)", 
    "Like foreach, but the values for which the result of 
    message are non-Nil are returned in a new List. Example:
    <pre>list(1, 5, 7, 2) select(i, v, v > 3) print
==> 5, 7

list(1, 5, 7, 2) select(v, v > 3) print
==> 5, 7</pre>")
    

    docSlot("detect(optionalIndex, value, message)", 
    "Returns the first value for which the message evaluates to a non-Nil. Example:
    <pre>list(1, 2, 3, 4) detect(i, v, v > 2)
==> 3

list(1, 2, 3, 4) detect(v, v > 2)
==> 3</pre>")
    

    docSlot("addIfAbsent(anObject)", 
    "Adds each value not already contained by the receiver, returns self. ")
    

    docSlot("appendSeq(aList1, aList2, ...)", 
    "Add the items in the lists to the receiver. Returns self.")
    

    docSlot("append(anObject1, anObject2, ...)", 
    """Appends the arguments to the end of the list. Returns self.""")
    

    docSlot("push(anObject1, anObject2, ...)", 
    "Same as add(anObject1, anObject2, ...).")
    

    docSlot("prepend(anObject1, anObject2, ...)", 
    "Inserts the values at the beginning of the list. Returns self.")
    

    docSlot("remove(anObject, ...)", 
    "Removes all occurrences of the arguments from the receiver. Returns self. ")
    

    docSlot("pop", 
    "Returns the last item in the list and removes it 
    from the receiver. Returns nil if the receiver is empty. ")
    

    docSlot("removeAt(index)", 
    "Removes the item at the specified index and returns the value removed. 
    Raises an exception if the index is out of bounds. ")
    

    docSlot("atPut(index, anObject)", 
    "Replaces the existing value at index with anObject. 
    Raises an exception if the index is out of bounds. Returns self.")
    

    docSlot("empty", 
    "Removes all items from the receiver.") 
    

    docSlot("swapIndices(index1, index2)", 
    "Exchanges the object at index1 with the object at index2. 
    Raises an exception if either index is out of bounds. Returns self.")
    

    docSlot("reverse", 
    "Reverses the ordering of all the items in the receiver. Returns self.") 
    

    docSlot("preallocateToSize(aNumber)", 
    "Preallocate array memory to hold aNumber number of items.")
    

    docSlot("sort(optionalExpression)", 
    "Sorts the list using the compare method on the items. Returns self.
    If an optionalExpression is provided, the sort is done on the result of the evaluation 
    of the optionalExpression on each value.")
    

    docSlot("sortBy(aBlock)", 
    "Sort the list using aBlock as the compare function. Returns self.")
    

    docSlot("selectInPlace(optionalIndex, value, message)", 
    "Like select, but the List is modified in-place. Returns self. ")
    

    docSlot("mapInPlace(optionalIndex, value, message)", 
    "Replace each value with the return of message. Returns self.")
    
)

Map ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")    
docObject("Map")
docDescription("A key/value dictionary appropriate for holding large key/value collections.")


    docSlot("empty", "Removes all keys from the receiver. Returns self.")
    

    docSlot("at(keyString, optionalDefaultValue)", 
    "Returns the value for the key keyString. Returns nil if the key is absent. ")
    

    docSlot("atPut(keyString, aValue)", 
    "Inserts/sets aValue with the key keyString. Returns self. ")
    

    docSlot("atIfAbsentPut(keyString, aValue)", 
    "Inserts/sets aValue with the key keyString if the 
    key is not already present. Returns self. ")
    

    docSlot("size", 
    "Returns the number of key/value pairs in the receiver.") 
    

    docSlot("hasKey(keyString)", 
    "Returns self if the key is present or Nil otherwise.")
    

    docSlot("removeAt(keyString)", 
    "Removes the specified keyString if present. Returns self. ")
    

    docSlot("hasValue(aValue)", 
    "Returns self if the value is one of the Map's values or Nil otherwise.")
    

    docSlot("keys", "Returns a List of the receivers keys.")
    

    docSlot("values", 
    "Returns a List of the receivers values.")
    

    docSlot("foreach(optionalKey, value, message)", 
    """For each key value pair, sets the locals key to 
    the key and value to the value and executes message.
    Example:
    <pre>aMap foreach(k, v, writeln(k, " = ", v))
aMap foreach(v, write(v))</pre>
    
    Example use with a block:
    
    <pre>myBlock = block(k, v, write(k, " = ", v, "\n")) 
aMap foreach(k, v, myBlock(k, v))</pre>""")
    
)

Message ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Message")
docDescription("""A Message object encapsulates the action of a message send. Blocks are composed of a Message and its children.

Terminology

Example;
A B(C D); E F
In the above example:
"A" is the "name" of the first message
the B message is "attached" to the A message
the C message is the first argument of the B message
the D message is attached the C message
the E message is the "next" message of the A message
the F message is attached the E message


Important; Modifying the message tree of a block currently in use may cause a crash if a garbage collection cycle occurs. If the implementation were changed to retain every called message, this could be avoided. But the cost to performance seems to outweigh the need to cover this case for now.
""")


    docSlot("lineNumber", 
    "Returns the line number of the message. The charcter number 
    is typically the line number in the source text from with the message was read. ")
    

    docSlot("setLineNumber(aNumber)", 
    "Sets the line number of the message. Returns self.")
    

    docSlot("characterNumber", 
    "Returns the message character number. The charcter number is typically 
    the beggining character index in the source text from with the message was read. ")
    

    docSlot("setCharacterNumber(aNumber)", 
    "Sets the character number of the message. Returns self.")
    

    docSlot("label", 
    "Returns the message label. The label is typically set the the 
    name of the file from which the source code for the message was read.")
    

    docSlot("setLabel(aString)", 
    "Sets the label of the message. Returns self.")
    

    docSlot("asString", 
    "Same as code().")
    

    docSlot("clone", 
    "Returns a Message that is a deep copy of the receiver. ")
    

    docSlot("name", 
    "Returns the name of the receiver. ")
    

    docSlot("setName(aString)", 
    "Sets the name of the receiver. Returns self. ")
    

    docSlot("code", 
    "Returns a String containing a decompiled code representation of the receiver.")
    

    docSlot("nextMessage", 
    "Returns the next message in the message chain or Nil if there is no next message. ")
    

    docSlot("setNextMessage(aMessageOrNil)", 
    "Sets the next message in the message chain to a deep copy of 
    aMessage or it removes the next message if aMessage is Nil. ")
    

    docSlot("attachedMessage", 
    "Returns the message attached to the receiver or Nil 
    if there is no message attached. ")
    

    docSlot("setAttachedMessage(aMessage)", 
    "Sets the message attached the message chain to a deep 
    copy of aMessage. If aMessage is Nil, it removes any attached message. ")
    

    docSlot("argAt(indexNumber)", 
    "Returns Message object for the specified argument or Nil if none exists.")
    

    docSlot("arguments", 
    "Returns a list of the message objects that act as the 
    receiver's arguments. Modifying this list will not alter the actual 
    list of arguments. Use the arguments_() method to do that. ")
    

    docSlot("setArguments(aListOfMessages)", 
    "Sets the arguments of the receiver to deep copies of 
    those contained in aListOfMessages.  Returns self.")
    

    docSlot("fromString(aString)", 
    "Returns a new Message object for the compiled(but not executed) 
    result of aString. ")
    

    docSlot("cachedResult", 
    "Returns the cached result of the Message or Nil if there is none.")
    

    docSlot("setCachedResult(anObject)", 
    "Sets the cached result of the message. Returns self.")
    

    docSlot("removeCachedResult", 
    "Removes the cached result of the Message.")
    

    docSlot("argsEvaluatedIn(anObject)", 
    "Returns a List containing the argument messages evaluated in the 
    context of anObject. ")
    
)

Nil ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Nil")
docDescription("""Nil is a singleton object that is used as a placeholder and to mean "false" in Io.""")


    docSlot("clone", 
    "returns self. (Nil is a singleton)")
    

    docSlot("and anObject", 
    "Returns Nil.")
    

    docSlot("or anObject", 
    "Returns anObject if anObject is not Nil. Otherwise returns Nil.")
    

    docSlot("print", 
    "Prints 'Nil'. Returns self.")
    

    docSlot("isNil", 
    "Returns Nop.")
    

    docSlot("ifNil(<message>)", 
    "Evaluates message.")
    
)

Number ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Number")
docInclude("_ioCode/Number.io")
docDescription("A container for a double (a 64bit floating point number on most platforms).")


    docSlot("htonl", 
    "Returns a new number with the first 4 bytes of the receiver switched from
    host to network byte order.")
    

    docSlot("htonl", 
    "Returns a new number with the first 4 bytes of the receiver switched from
    network to host byte order.")
    

    docSlot("asNumber", 
    "Returns self.")
    

    docSlot("+(aNumber)", 
    "Returns a new number that is the sum of the receiver and aNumber.")
    

    docSlot("-(aNumber)", 
    "Returns a new number that is the difference of the receiver and aNumber.")
    

    docSlot("/(aNumber)", 
    "Returns a new number with the value of the receiver diveded by aNumber.")
    

    docSlot("*(aNumber)", 
    "Returns a new number that is the product of the receiver and aNumber.")
    

    docSlot("print", "Prints the number.")
    

    docSlot("asCharacter", 
    "Returns a String containing a single character whose 
    value is the ascii value of the first byte of the receiver.")
    

    docSlot("asBuffer(optionalNumberOfBytes)", 
    "Returns a Buffer containing a the number of bytes specified by 
    optionalNumberOfBytes (up to the size of a double on the platform) of the reciever. 
    If no optionalNumberOfBytes is specified, it is assumed to be the number of bytes 
    in a double on the host platform.")

    

    docSlot("asString(optionalIntegerDigits, optionalFactionDigits)", 
    """Returns a string representation of the receiver. For example:
    <pre>1234.5678 asString(0, 2)</pre>
    would return:
    <pre>$1234.56</pre>
    """)
    

    docSlot("abs", 
    "Returns a number with the absolute value of the receiver.")
    

    docSlot("acos", 
    "Returns a number with the arc cosine of the receiver.")
    

    docSlot("asin", 
    "Returns a number with the arc sine of the receiver.")
    

    docSlot("atan", 
    "Returns a number with the arc tangent of the receiver.")
    

    docSlot("atan2(aNumber)", 
    "Returns a number with the arc tangent of y/x where y is the receiver and x is aNumber.")
    

    docSlot("ceil", 
    "Returns the a number with the receiver's value rounded up to 
    the nearest integer if it's fractional component is greater than 0.")
    

    docSlot("cos", 
    "Returns the cosine of the receiver.")
    

    docSlot("exp", 
    "Returns e to the power of the receiver.")
    

    docSlot("factorial", 
    "Returns the factorial of the receiver.")
    

    docSlot("floor", 
    "Returns the a number with the receiver's value rounded 
    down to the nearest integer if it's fractional component is not 0.")
    

    docSlot("log", 
    "Returns the natural logarithm of the receiver.")
    

    docSlot("log10", 
    "Returns the base 10 logarithm of the receiver.")
    

    docSlot("max(aNumber)", 
    "Returns the greater of the receiver and aNumber.")
    

    docSlot("min(aNumber)", 
    "Returns the lesser of the receiver and aNumber.")
    

    docSlot("%(aNumber)", 
    "Returns the receiver modulus aNumber.")
    

    docSlot("pow(aNumber)", 
    "Returns the value of the receiver to the aNumber power.")
    

    docSlot("roundDown", 
    "Returns the a number with the receiver's value rounded 
    down to the nearest integer if it's fraction component is <= .5.")
    

    docSlot("roundUp", 
    "Returns the a number with the receiver's value rounded up to 
    the nearest integer if it's fraction component is >= .5.")
    

    docSlot("sin", 
    "Returns the sine of the receiver.")
    

    docSlot("sqrt", 
    "Returns the square root of the receiver.")
    

    docSlot("sqrt", 
    "Returns the square root of the receiver.")
    

    docSlot("cubed", 
    "Returns the cube of the receiver.")
    

    docSlot("tan", 
    "Returns the tangent of the receiver.")
    

    docSlot("toggle", "Returns 1 if the receiver is 0. Returns 0 otherwise.")
    

    docSlot("&(aNumber)", "Returns a new number with the bitwise AND of the receiver and aNumber.")
    

    docSlot("|(aNumber)", "Returns a new number with the bitwise OR of the receiver and aNumber.")
    

    docSlot("bitwiseXor(aNumber)", 
    "Returns a new number with the bitwise XOR of the receiver and aNumber.")
    

    docSlot("bitwiseComplement", 
    "Returns a new number with the bitwise complement of the 
    receiver. (Turns the 0 bits of become 1s and the 1 bits become 0s. )")
    

    docSlot("shiftLeft(aNumber)", 
    "Shifts the bits of the receiver left by the number of places specified by aNumber.")
    

    docSlot("shiftRight(aNumber)", 
    "Shifts the bits of the receiver right by the number of places specified by aNumber.")
    

    docSlot("isEven", 
    "Returns self if integer form of the receiver is even. Otherwise returns Nil.")
    

    docSlot("isOdd", 
    "Returns self if integer form of the receiver is even. Otherwise returns Nil.")
    

    docSlot("isAlphaNumeric", 
    "Returns self if the receiver is an alphanumeric 
    character value. Otherwise returns Nil.")
    

    docSlot("isLetter", 
    "eturns self if the receiver is an alphanetic character value. Otherwise returns Nil.")
    

    docSlot("isControlCharacter", 
    "Returns self if the receiver is an control 
    character value. Otherwise returns Nil.")
    

    docSlot("isDigit", 
    "Returns self if the receiver is an numeric 
    digit character value. Otherwise returns Nil.")
    

    docSlot("isGraph", 
    "Returns self if the receiver is a printing character 
    value except space. Otherwise returns Nil.")
    

    docSlot("isLowerCase", 
    "Returns self if the receiver is an lower case 
    character value. Otherwise returns Nil.")
    

    docSlot("isUpperCase", 
    "Returns self if the receiver is an upper case 
    character value. Otherwise returns Nil.")
    

    docSlot("isPrint", 
    "Returns self if the receiver is an printing character 
    value, including space. Otherwise returns Nil.")
    

    docSlot("isPunctuation", 
    "Returns self if the receiver is an printing character 
    value, except space letter or digit. Otherwise returns Nil.")
    

    docSlot("isSpace", 
    "Returns self if the receiver is a space, formfeed, 
    newline carriage return, tab or vertical tab character value. Otherwise returns Nil.")
    

    docSlot("isHexDigit", 
    "Returns self if the receiver is hexidecimal digit 
    character value. Otherwise returns Nil.")
    

    docSlot("lowerCase", 
    "Returns a new Number containing a lower case version of the receiver.")
    

    docSlot("upperCase", 
    "Returns a new Number containing a upper case version of the receiver.")
    

    docSlot("between(aNumber1, aNumber2)", 
    "Returns the receiver if the receiver's value is between or
    equal to aNumber1 and aNumber2, otherwise returns nil.")
    

    docSlot("clip(aNumber1, aNumber2)", 
    "Returns self if the receiver is between aNumber1 and aNumber2. 
    Returns aNumber1 if it is less than aNumber1. Returns aNumber2 if it is greater than aNumber2.")
    

    docSlot("negate", 
    "Returns new number that is negated version of the receiver.")
    

    docSlot("at(bitIndexNumber)", 
    "Returns a new Number containing 1 if the receiver cast to a long 
    has it's  bit set to 1 at bitIndexNumber. Otherwise returns 0.")
    

    docSlot("integerMax", "Returns the maximum integer value.")
    

    docSlot("integerMin", "Returns the minimum integer value.")
    

    docSlot("longMax", "Returns the maximum long value.")
    

    docSlot("longMin", 
    "Returns the minimum long value.")
    

    docSlot("shortMax", "Returns the maximum short value.")
    

    docSlot("shortMin", "Returns the minimum short value.")
    

    docSlot("unsignedLongMax", "Returns the maximum unsigned long value.")
    

    docSlot("unsignedIntMax", "Returns the maximum unsigned int value.")
    

    docSlot("unsignedShortMax", "Returns the minimum unsigned int value.")
    

    docSlot("floatMax", "Returns the maximum float value.")
    

    docSlot("floatMin", "Returns the minimum float value.")
    

    docSlot("floatMax", "Returns the maximum double precision float value.")
    

    docSlot("doubleMin", "Returns the minimum double precision float value.")
    

    docSlot("repeatTimes(<message>)", 
    "Evaluates message a number of times that corresponds to the receivers 
    integer value. This is significantly  faster than a for() or while() loop.")
    
)

Object ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Object")
docInclude("_ioCode/Object.io")
docDescription("""An Object is a key/value dictionary with string keys and values of any type. The prototype Object contains a clone slot that is a CFuntion that creates new objects. When cloned, an Object will call it's init slot (with no arguments).

<p>
<b>Important note:</b> <p>The data structure used for objects is optimized for objects with relatively few slots (less than 100). Objects become very memory inefficient (exponentially so) when they have a large number of slots. Objects should not be used as large hash tables or lists. Use a Hash or List primitive instead.""")


    docSlot("proto", 
    "Same as; method(self protos first)")
    

    docSlot("setProto(anObject)", 
    "Sets the first proto of the receiver to anObject, replacing the 
    current one, if any. Returns self.")
    

    docSlot("appendProto(anObject)", 
    "Appends anObject to the receiver's proto list. Returns self.")
    

    docSlot("prependProto(anObject)", 
    "Prepends anObject to the receiver's proto list. Returns self.")
    

    docSlot("removeProto(anObject)", 
    "Remove's anObject from the receiver's proto list if it 
    is present. Returns self.")
    

    docSlot("removeAllProtos", 
    "Remove's all of the receiver's protos. Returns self. ")
    

    docSlot("setProtos(aList)", 
    "Replaces the receiver's protos with a copy of aList. Returns self.") 
    

    docSlot("protos", 
    "Returns a copy of the receiver's protos list.")
    

    docSlot("compare(anObject)", 
    "Returns a number containing the comparison value of the target with anObject.")
    

    docSlot("hasProto(anObject)", 
    "Returns self if anObject is found in the proto 
    path of the target, Nil otherwise.")
    

    docSlot("print", 
    "Prints a string representation of the object. Returns Nil.")
    

    docSlot("perform(methodName, <arg1>, <arg2>, ...)", 
    "Performs the method corresponding to methodName with the arguments supplied.") 
    

    docSlot("performWithArgList(methodName, argList)", 
    "Performs the method corresponding to methodName with the arguments in the argList. ")
    

    docSlot("write(<any number of arguments>)", 
    "Sends a print message to the evaluated result of each argument. Returns Nil.")
    

    docSlot("writeln(<any number of arguments>)", 
    "Same as write() but also writes a return character at the end. Returns Nil.")
    

    docSlot("clone ", "Returns a clone of the receiver.")
    

    docSlot("shallowCopy", 
    "Returns a shallow copy of the receiver.")
    

    docSlot("setSlot(slotNameString, valueObject)", 
    "Sets the slot slotNameString in the receiver to 
    hold valueObject. Returns valueObject.")
    

    docSlot("updateSlot(slotNameString, valueObject)", 
    "Same as setSlot(), but raises an error if the slot does not 
    already exist in the receiver's slot lookup path.")
    

    docSlot("getSlot(slotNameString)", 
    "Returns the value of the slot named slotNameString 
    (following the lookup path) or Nil if no such slot is found.")
    

    docSlot("hasSlot(slotNameString)", 
    "Returns self if the slot exists in the receiver or Nil otherwise.")
    

    docSlot("removeSlot(slotNameString)", 
    "Removes the specified slot (only) in the receiver if it exists. Returns self.")
    

    docSlot("slotNames", 
    "Returns a list of strings containing the names of the 
    slots in the receiver (but not in it's lookup path).")
    

    docSlot("while(<condition>, <message>)", 
    "Keeps evaluating message until condition return Nil. 
    Returns the result of the last message evaluated or Nil if none were evaluated.")
    

    docSlot("loop(<message>)", 
    "Keeps evaluating message until a break.")
    

    docSlot("for(<counter>, <start>, <end>, <do message>)", 
    "See manual.html. ")
    

    docSlot("if(<condition>, <trueMessage>, <optionalFalseMessage>)", 
    "Evaluates trueMessage if condition evaluates to a non-Nil. 
    Otherwise evaluates optionalFalseMessage if it is present. 
    Returns the result of the evaluated message or Nil if none was evaluated.")
    

    docSlot("return anObject", 
    "Return anObject from the current execution block.")
    

    docSlot("break(optionalReturnValue)", 
    "Break the current loop, if any. ")
    

    docSlot("continue", 
    "Skip the rest of the current loop iteration and start on 
    the next, if any. ")
    

    docSlot("ancestorWithSlot(slotName)", 
    "Returns the first ancestor of the receiver that contains 
    a slot of the specified name or Nil if none is found.")
    

    docSlot("contextWithSlot(slotName)", 
    "Returns the first context (starting with the receiver and following the lookup path) 
    that contains a slot of the specified name or Nil if none is found.")
    

    docSlot("doMessage(aMessage, optionalContext)", 
    "Evaluates the message object in the context of the receiver. 
    Returns the result. optionalContext can be used to specific the locals 
    context in which the message is evaluated.")
    

    docSlot("doString(aString)", 
    "Evaluates the string in the context of the receiver. Returns the result. ")
    

    docSlot("doFile(pathString)", 
    "Evaluates the File in the context of the receiver. Returns the result. ")
    

    docSlot("== aValue", 
    "Returns the receiver is aValue is identical, Nil otherwise. ")
    

    docSlot("!= aValue", 
    "Returns the receiver is aValue is not identical, nil otherwise. ")
    

    docSlot("foreach([name,] value, message)", 
    """For each slot, set name to the slot's 
    name and value to the slot's value and execute message. Example:
    <pre>
    myObject foreach(n, v, 
	writeln("slot ", n, " = ", v type)
    )
    myObject foreach(v, 
	writeln("slot type ", v type)
    )
    </pre>
    """)
    

    docSlot("- (aNumber)", 
    "Returns the negative version of aNumber. 
    Raises an exception if argument is not a number.")
    

    docSlot("self ", 
    "Returns self.")
    

    docSlot("type", 
    "Returns a string containing the name of the type of Object (Number, String, etc).")
    

    docSlot("then (<message>)", 
    "Evaluates the argument and returns Nop.")
    

    docSlot("(<message>)", 
    """The "" method evaluates the argument and returns the result.""")
    

    docSlot("else (<message>)", 
    "Returns self. (Does not evaluate argument)")
    

    docSlot("and(<message>)", 
    "Evaluates argument and returns Nil if it is Nil or self otherwise.")
    

    docSlot("or(<message>)", 
    "Returns self.")
    

    docSlot("< (<message>)", 
    "Evaluates argument and returns self if self is less or Nil if not.")
    

    docSlot("<= (<message>)", 
    "Evaluates argument and returns self if self is less 
    than or equal to it, or Nil if not.")
    

    docSlot("> (<message>)", 
    "Evaluates argument and returns self if self is greater than it, or Nil if not.")
    

    docSlot(">= (<message>)", 
    "Evaluates argument and returns self if self is greater 
    than or equal to it, or Nil if not.")
    

    docSlot("uniqueId", 
    "Returns a Number containing a unique id for the receiver.")
    

    docSlot("do(<message>)", 
    "Evaluates the message in the context of the receiver. Returns self.")
    

    docSlot("message(<message>)", 
    "Return the message object for the argument or Nil if there is no argument.")
    

    docSlot("isNil", 
    "Returns Nil.")
    

    docSlot("ifNil(<message>)", 
    "Returns self. (Does not evaluate the argument)")
    

    docSlot("ifTrue(<message>)", 
    "Evaluates argument and returns result.")
    

    docSlot("ifFalse(<message>)", 
    "Returns Nil. (Does not evaluate the argument)")
    

    docSlot("tailCall(<message>)", 
    """Does a tailCall on the currently executing Block. Example:
    
    <pre>
    Io> a := method(x, x = x + 1; if(x > 10, return x); tailCall(x))
    ==> method(x, updateSlot("x", x +(1));
    if(x >(10), return(x));
    tailCall(x))
    Io> a(1)
    ==> 11
    </pre>
    """)
    

    docSlot("turnOnMessageDebugging", 
    "Turns on message level debugging for this actor. When on, this 
    actor will send a vmWillSendMessage message to the Debugger object before 
    each message send and pause itself. See the Debugger object documentation 
    for more information. ")
    

    docSlot("turnOffMessageDebugging", 
    "Turns off message level debugging for this actor. ")
    

    docSlot("actorCallStack", 
    "Returns a copy of the receivers call stack if it is 
    an actor, or Nil, otherwise. ")
    

    docSlot("setIsActivatable(aValue)", 
    "When called with a non-Nil aValue, sets the object 
    to call it's activate slot when accessed as a value. Turns this behavior 
    off if aValue is Nil. Only works on Objects which are not Activatable 
    Primitives (such as CFunction or Block). Returns self.")
    

    docSlot("deprecated", 
    """Prints "The slot [slotName] has been deprecated".""")
    
)

Object ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")

)

Object ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")

)

Random ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Random")
docDescription("""A high quality and reasonably fast random number generator based on Makoto Matsumoto, Takuji Nishimura, and Eric Landry's implementation of the Mersenne Twister algorithm.
<p>
Reference: 
<p>
<i>
M. Matsumoto and T. Nishimura, <br>
"Mersenne Twister: A 623-Dimensionally Equidistributed Uniform Pseudo-RandomGen Number
Generator", <br>
ACM Transactions on Modeling and Computer Simulation, Vol. 8, No. 1, January 1998, pp 3--30.
</i>""")


    docSlot("value(optionalArg1, optionalArg2)", 
    "If called with:
    <ul>
    <li> no arguments, it returns a floating point 
    random Number between 0 and 1.
    <li> one argument, it returns a floating point random 
    Number between 0 and optionalArg1.
    <li> two arguments, it returns a floating point random 
    Number between optionalArg1 and optionalArg2.
    </ul>
    ")
    

    docSlot("setSeed(aNumber)", 
    "Sets the random number generator seed to the unsign int version of aNumber.")
    
)

Sandbox ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Sandbox")
docDescription("Sandbox can be used to run separate instances of Io within the same process.")


    docSlot("makeSecure", 
    "Removes Collector DynLib File Directory Debugger Store Sandbox objects.")
    

    docSlot("messageCount", 
    "Returns a number containing the messageCount limit of the Sandbox. ")
    

    docSlot("setMessageCount(anInteger)", 
    "Sets the messageCount limit of the receiver. ")
    

    docSlot("timeLimit", 
    "Returns a number containing the time limit of calls made to the Sandbox. ")
    

    docSlot("setTimeLimit(aDouble)", 
    "Sets the time limit of the Sandbox. ")
    

    docSlot("doSandboxString(aString)", 
    "Evaluate aString instead the Sandbox. ")
    
)

Scheduler ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Scheduler")
docDescription("Contains methods related to the IoVM coroutine scheduler.")


    docSlot("actors", 
    "Returns a copy of the list of all active actors.")
    

    docSlot("currentActor", 
    "Returns the current actor object.")
    

    docSlot("activeActorCount", 
    "Returns the number of active actors.")
    

    docSlot("setSleepInterval(aNumber)", 
    "Sets the sleep interval (in seconds) per scheduler cycle. 
    The actual amount of time slept each yield is this interval divided by the
    number of active actors at the moment of the yield. In this way, the total 
    time slept per scheduler cycle should typically be close to the sleepInterval. 
    Return self.")
    

    docSlot("sleepInterval", 
    "Returns a number containing the sleep interval (in seconds)
    between actor coroutine switches. ")
    
)

ImmutableSequence ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("ImmutableSequence")
docDescription("""A Sequence is a container for a list of data elements. Immutable Sequences are also called "Symbols". """)


    docSlot("with(aSequence, ...)", 
    "Returns a new Sequence which is the concatination of the arguments. 
    The returned sequence will have the same mutability status as the receiver.")
    

    docSlot("asBinaryNumber", 
    "Returns a Number containing the first 8 bytes of the 
    receiver without casting them to a double.")
    

    docSlot("asSymbol", 
    "Returns a immutable Sequence (aka Symbol) version of the receiver.")
    

    docSlot("isSymbol", 
    "Returns self if the receiver is a 
    immutable Sequence (aka, a Symbol) or Nil otherwise.")
    

    docSlot("isMutable", 
    "Returns self if the receiver is a mutable Sequence or Nil otherwise.")
    

    docSlot("print", 
    "Prints the receiver as a string. Returns self.")
    

    docSlot("linePrint", 
    "Prints the Sequence and a newline character.")
    

    docSlot("size", 
    "Returns the length in bytes of the receiver.")
    

    docSlot("at(aNumber)", 
    "Returns a value at the index specified by aNumber. 
    Raises an error if the index is out of bounds.")
    

    docSlot("slice(startIndex, endIndex)", 
    "Returns a new string containing the subset of the 
    receiver from the startIndex to the endIndex. The endIndex argument 
    is optional. If not given, it is assumed to be the end of the string. ")
    

    docSlot("between(aSequence, anotherSequence)", 
    "Returns a new Sequence containing the bytes between the 
    occurance of aSequence and anotherSequence in the receiver.")
    

    docSlot("findSeq(aSequence, optionalStartIndex)", 
    "Returns a number with the first occurrence of aSequence in 
    the receiver after the startIndex. If no startIndex is specified, 
    the search starts at the beginning of the buffer. 
    Nil is returned if no occurences are found. ")
    

    docSlot("reverseFindSeq(aSequence, startIndex)", 
    "Returns a number with the first occurrence of aSequence in 
    the receiver before the startIndex. The startIndex argument is optional. 
    By default reverseFind starts at the end of the string. Nil is 
    returned if no occurrences are found. ")
    

    docSlot("beginsWithSeq(aSequence)", 
    "Returns self if the receiver begins with aSequence, Nil otherwise.") 
    

    docSlot("endsWithSeq(aSequence)", 
    "Returns self if the receiver ends with aSequence, Nil otherwise. ")
    

    docSlot("join(aList)", 
    """Joins a list of strings and returns a single string 
    result. Example,

    <pre>
    l = List clone append("a", "b", "c")
    result = Sequence join(l) // returns "abc"</pre>

    If the receiver has a value, it becomes the separating string 
    that is inserted between the strings in the result. Example,
    <pre>
    l = List clone append("a", "b", "c")
    result = "." join(l) // result = "a.b.c"</pre>
    """)
    

    docSlot("containsSeq(aSequence)", 
    "Returns self if the receiver contains the substring 
    aSequence, Nil otherwise. ")
    

    docSlot("containsAnyCaseSeq(aSequence)", 
    "Returns self if the receiver contains the aSequence 
    regardless of casing, Nil otherwise. ")
    

    docSlot("isLowerCase", 
    "Returns self if all the characters in the string are lower case.")
    

    docSlot("isUpperCase", 
    "Returns self if all the characters in the string are upper case.")
    

    docSlot("isEqualAnyCase(aSequence)", 
    "Returns self if aSequence is equal to the receiver 
    ignoring case differences. Returns Nil otherwise.")
    

    docSlot("asNumber", 
    "Returns the receiver converted to a number. 
    Initial whitespace is ignored.")
    

    docSlot("split(optionalArg1, optionalArg2,...) ", 
    """Returns a List containing the sub-sequences of the 
    receiver divided by the given arguments. Example: 
    <pre>"a, b, c" split(",", " ") == list("a", "b", "c")</pre>""")
    

    docSlot("signedInt32At(indexNumber)", 
    "Treats the buffer as an array of 32 bit signed integers 
    and returns the Number at the specified index.")
    

    docSlot("unsignedInt32At(indexNumber)", 
    "Treats the buffer as an array of 32 bit unsigned 
    integers and returns the Number at the specified index.")
    

    docSlot("float32At(indexNumber)", 
    "Treats the buffer as an array of 32 bit floats and 
    returns the Number at the specified index.")
    

    docSlot("fromBase(aNumber)", 
    "Returns a number with a base 10 representation of the receiver 
    converted from the specified base. Only base 2 through 32 are currently supported.")
    

    docSlot("toBase(aNumber)", 
    "Returns a Sequence containing the receiver(which is 
    assumed to be a base 10 number) converted to the specified base. 
    Only base 8 and 16 are currently supported. ")
    

    docSlot("foreach(optionalIndex, value, message)", 
    """For each element, set index to the index of the 
    element and value the element value and execute message. Example:
    <pre>
    aSequence foreach(i, v, writeln("value at index ", i, " is ", v))
    aSequence foreach(v, writeln("value ", v))
    </pre>""")
    

    docSlot("asMessage", 
    "Returns the compiled message object for the string.")
    

    docSlot("..(aSequence)", 
    "Returns a copy of the receiver with aSequence appended to it. ")
    

    docSlot("asMutable", 
    "Returns a mutable copy of the receiver. ")
    

    docSlot("asUppercase", 
    "Returns a symbol containing the reveiver made uppercase. ")
    

    docSlot("asLowercase", 
    "Returns a symbol containing the reveiver made lowercase. ")
    

    docSlot("lastPathComponent", 
    "Returns a string containing the receiver clipped up 
    to the last path separator. ")
    

    docSlot("pathExtension", 
    "Returns a string containing the receiver clipped up to the last period. ")
    

    docSlot("fileName", 
    "Returns the last path component sans the path extension.")
    

    docSlot("cloneAppendPath(aSequence)", 
    "Appends argument to a copy the receiver such that there is one 
    and only one path separator between the two and returns the result.")
    

    docSlot("pathComponent", 
    "Returns a slice of the receiver before the last path separator as a symbol. ")
    

    docSlot("beforeSeq(aSequence)", 
    "Returns the slice of the receiver (as a Symbol) before 
    aSequence or self if aSequence is not found.")
    

    docSlot("afterSeq(aSequence)", 
    "Returns the slice of the receiver (as a Symbol) 
    after aSequence or Nil if aSequence is not found. 
    If aSequence is empty, the receiver (or a copy of the 
    receiver if it is mutable) is returned.")
    

    docSlot("asCapitalized", 
    "Returns a copy of the receiver with the first charater made uppercase.")
    

    docSlot("asCapitalized", 
    "Returns a copy of the receiver with the first charater made uppercase.")
    
)

Sequence ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Sequence")
docDescription("""A Sequence is a container for a list of data elements. Typically these elements are each 1 byte in size. A Sequence can be either mutable or immutable. When immutable, only the read-only methods can be used. 
<p>
Terminology
<ul>
<li> Buffer: A mutable Sequence of single byte elements, typically in a binary encoding
<li> Symbol or String: A unique immutable Sequence, typically in a character encoding
</ul>
""")


    docSlot("appendSeq(aSequence1, aSequence2, ...)", 
    "Appends aSequence arguments to the receiver. Returns self. ")
    

    docSlot("append(aNumber)", 
    "Appends aNumber (cast to a byte) to the receiver. 
    Returns self. ")
    

    docSlot("atInsertSeq(indexNumber, aSequence)", 
    "Returns a new Sequence with aSequence inserted at 
    indexNumber in the receiver.  ")
    

    docSlot("removeSlice(startIndex, endIndex)", 
    "Removes the bytes from startIndex to endIndex. 
    Returns self.")
    

    docSlot("removeLastByte", 
    "Removes the last byte from the receiver. Returns self.")
    

    docSlot("setSize(aNumber)", 
    "Sets the length in bytes of the receiver to aNumber. Return self.")
    

    docSlot("preallocateToSize(aNumber)", 
    "If needed, resize the memory alloced for the receivers 
    byte array to be large enough to fit the number of bytes specified by 
    aNumber. This is useful for preallocating the memory so it doesn't 
    keep getting allocated as the Sequence is appended to. This operation 
    will not change the Sequence's length or contents. Returns self.")
    

    docSlot("replaceSeq(aSequence, anotherSequence)", 
    "Returns a new Sequence with all occurances of aSequence 
    replaced with anotherSequence in the receiver. Returns self.")
    

    docSlot("replaceFirstSeq(aSequence, anotherSequence, optionalStartIndex)", 
    "Returns a new Sequence with the first occurance of aSequence 
    replaced with anotherSequence in the receiver. If optionalStartIndex is 
    provided, the search for aSequence begins at that index. Returns self.")
    

    docSlot("atPut(aNumberIndex, aNumberByte)", 
    "Sets the byte at the index specified by aNumberIndex 
    to the byte value of aNumberByte. Returns self. ")
    

    docSlot("lowercase", 
    "Returns a copy of the receiver with all characters made lowercase. ")
    

    docSlot("uppercase", 
    "Returns a copy of the receiver with all characters made uppercase. ")
    

    docSlot("clipBeforeSeq(aSequence)", 
    "Clips receiver before aSequence.")
    

    docSlot("clipAfterSeq(aSequence)", 
    "Removes the contents of the receiver after the end of 
    the first occurance of aSequence. Returns self if anything was 
    removed, or Nil otherwise.")
    

    docSlot("clipBeforeEndOfSeq(aSequence)", 
    "Removes the contents of the receiver before the end of 
    the first occurance of aSequence. Returns self if anything was 
    removed, or Nil otherwise.")
    

    docSlot("clipAfterStartOfSeq(aSequence)", 
    "Removes the contents of the receiver after the beginning of 
    the first occurance of aSequence. Returns self if anything was 
    removed, or Nil otherwise.")
    

    docSlot("clear", 
    "Sets all bytes in the receiver to 0x0 and sets 
    it's length to 0. Returns self.")
    

    docSlot("removeOddIndexes", 
    "Removes odd indexes in the receiver. 
    For example, list(1,2,3) removeOddIndexes == list(2). Returns self.")
    

    docSlot("removeEvenIndexes", 
    "Removes even indexes in the receiver. 
    For example, list(1,2,3) removeEvenIndexes == list(1, 3). Returns self.")
    

    docSlot("duplicateIndexes", 
    "Duplicates all indexes in the receiver. 
    For example, list(1,2,3) duplicateIndexes == list(1,1,2,2,3,3). Returns self.")
    

    docSlot("replaceMap(aMap)", 
    "In the receiver, the keys of aMap replaced with it's values. Returns self.")
    

    docSlot("strip(optionalSequence)", 
    """Trims the whitespace (or optionalSequence) off both ends:
    <pre>
    "   Trim this string   \r\n" strip
    ==> "Trim this string"
    </pre>
    """)
    

    docSlot("lstrip(aSequence)", 
    """Strips the characters in aSequence 
    stripped from the beginning of the receiver. Example:
    <pre>
    "Keep the tail" lstrip(" eKp")
    ==> "the tail"
    </pre>
    """)
    

    docSlot("rstrip(aSequence)", 
    """Strips the characters in 
    aSequence stripped from the end of the receiver. Example:
    <pre>
    "Cut the tail off" rstrip(" afilot")
    ==> "Cut the"
    </pre>
    """)
    

    docSlot("setFloat32At(value, indexNumber)", 
    "Treats the buffer as an array of 32 bit floats and 
    returns the Number at the specified index.")
    

    docSlot("escape", 
    """Escape characters in the receiver are replaced with escape codes. 
    For example a string containing a single return character would contain the 
    following 2 characters after being escaped: "\n". Returns self.""")
    

    docSlot("unescape", 
    "Escape codes replaced with escape characters. Returns self.")
    

    docSlot("removePrefix(aSequence)", 
    "If the receiver begins with aSequence, it is removed. Returns self.")
    

    docSlot("removeSuffix(aSequence)", 
    "If the receiver end with aSequence, it is removed. Returns self.")
    

    docSlot("capitalize", 
    "First charater of the receiver is made uppercase.")
    

    docSlot("appendPathSeq(aSeq)", 
    "Appends argument to the receiver such that there is one 
    and only one path separator between the two. Returns self.")
    
)

Store ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Store")
docDescription("This object can be used to store the Io state in a way similar to a Smalltalk system image. However, objects are stored as individual indexed key/value records, so it opens the possibility of lazy loading and using the Io namespace itself as a high performance, transactional database.")

)

System ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("System")
docDescription("Contains methods related to the IoVM.")


    docSlot("exit(optionalReturnCodeNumber)", 
    "Shutdown the IoState (free all objects) and return 
    control to the calling program (if any). ")
    

    docSlot("getenv(nameString)", 
    "Returns a string with the value of the environment 
    variable whose name is specified by nameString.") 
    

    docSlot("system(aString)", 
    "Makes a system call and returns a Number for the return value.")
    
)

WeakLink ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("WeakLink")
docDescription("""A WeakLink is a primitive that can hold a reference to an object without preventing the garbage collector from collecting it. The link reference is set with the setLink() method. After the garbage collector collects an object, it informs any WeakLink objects whose link value pointed to that object. If a WeakLink has a delegate set, it will send the delagate a collectedWeakLink() message with the WeakLink as an argument.
<p>
The identifier value of a WeakLink is usefull for storing miscellaneous information about the link if needed.
<p>

<b>What are weak links useful for?</b>
<p>

In general, their handy when you want to monitor or cache an object without owning it. Examples:
<p>

<b>Distributed Objects example</b>
<p>

The original motivation for adding weaklinks was to deal with garbage collection in a distributed object system. In such a system, it can be usefull to keep a list of proxies that have been created to represent remote objects, but you don't want this list to cause those objects to be forever retained.
<p>

<b>Database Caching example</b>
<p>

In a persistance layer to a database it's often desirable to have separate queries return the same row objects if some other part of the application has already fetched them. This involves a cache, and weak links allow the cache to only keep database items that are still referenced by other parts of the application.
<p>

<b>Implementation notes</b>
<p>

When a WeakLink's link is set, it sets a weak link bit in the mark byte of the object being monitored. When the garbage collector collects an object, it checks this bit and if it's marked, looks in it's list of WeakLinks and sends a collected message to each WeakLink that references that object.
""")


    docSlot("setLink(aValue)", 
    "Sets the link pointer. Returns self.")
    

    docSlot("link", 
    "Returns the link pointer or Nil if none is set.")
    

    docSlot("setDelegate(aValue)", 
    "Sets the delegate. Returns self. ")
    

    docSlot("delegate", 
    "Returns the delegate or Nil if none is set.")
    

    docSlot("setIdentifier(aValue)", 
    "Sets the identifier. Returns self.")
    

    docSlot("identifier", 
    "Returns the identifier or Nil if none is set.")
    
)
